
from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .custom_fields import CustomFields


@JsonMap(
    {
        "ack_report": "ackReport",
        "ack_status": "ackStatus",
        "action_type": "actionType",
        "agency_code": "agencyCode",
        "app_receiver_id": "appReceiverID",
        "app_sender_id": "appSenderID",
        "atom_id": "atomId",
        "connector_name": "connectorName",
        "connector_type": "connectorType",
        "custom_fields": "customFields",
        "date_processed": "dateProcessed",
        "document_index": "documentIndex",
        "error_message": "errorMessage",
        "execution_id": "executionId",
        "functional_id": "functionalID",
        "gs_control": "gsControl",
        "gs_date": "gsDate",
        "gs_time": "gsTime",
        "gs_version": "gsVersion",
        "id_": "id",
        "isa_ack_report": "isaAckReport",
        "isa_ack_status": "isaAckStatus",
        "isa_control": "isaControl",
        "operation_name": "operationName",
        "outbound_validation_report": "outboundValidationReport",
        "outbound_validation_status": "outboundValidationStatus",
        "receiver_id": "receiverID",
        "receiver_id_qualifier": "receiverIDQualifier",
        "sender_id": "senderID",
        "sender_id_qualifier": "senderIDQualifier",
        "st_control": "stControl",
        "standard_id": "standardID",
        "test_indicator": "testIndicator",
        "transaction_set": "transactionSet",
    }
)
class X12ConnectorRecord(BaseModel):
    """X12ConnectorRecord

    :param account: The ID of the account in which this record ran.
    :type account: str
    :param ack_report: The acknowledgment report., defaults to None
    :type ack_report: str, optional
    :param ack_status: The acknowledgment status — either Accepted, Accepted with Errors, Partially Accepted, or Rejected., defaults to None
    :type ack_status: str, optional
    :param action_type: The type of action with which this record corresponds — Send for an outbound interchange, Get for an inbound interchange using the Disk, FTP, or SFTP communication method, or Listen for an inbound interchange using the AS2 or HTTP communication method.
    :type action_type: str
    :param agency_code: The Responsible Agency Code., defaults to None
    :type agency_code: str, optional
    :param app_receiver_id: For inbound interchanges, the Application Receiver’s Code, which identifies the receiver., defaults to None
    :type app_receiver_id: str, optional
    :param app_sender_id: For outbound interchanges, the Application Sender’s Code, which identifies the sender., defaults to None
    :type app_sender_id: str, optional
    :param atom_id: The ID of the Runtime that processed this record.
    :type atom_id: str
    :param connector_name: The value is Trading Partner for an X12 trading partner Send operation, or Start for an X12 trading partner Listen operation.
    :type connector_name: str
    :param connector_type: x12 is the connector type for any record.
    :type connector_type: str
    :param custom_fields: custom_fields
    :type custom_fields: CustomFields
    :param date_processed: The processing date and time for this record. The format is *yyyy-MM-dd'T'HH:mm:ss'Z'*, for example, 2016-01-31T15:32:00Z.
    :type date_processed: str
    :param document_index: The numerical index of this record in the run., defaults to None
    :type document_index: int, optional
    :param error_message: Any error message associated with this record.
    :type error_message: str
    :param execution_id: The ID of the run.
    :type execution_id: str
    :param functional_id: The identifier for the type of message in the functional group., defaults to None
    :type functional_id: str, optional
    :param gs_control: The group control number., defaults to None
    :type gs_control: str, optional
    :param gs_date: The preparation date of the interchange., defaults to None
    :type gs_date: str, optional
    :param gs_time: The preparation time of the interchange., defaults to None
    :type gs_time: str, optional
    :param gs_version: The Version, Release, and Industry identifier code., defaults to None
    :type gs_version: str, optional
    :param id_: The ID of this record.
    :type id_: str
    :param isa_ack_report: The interchange acknowledgment report, which contains descriptions of interchange segment validation errors., defaults to None
    :type isa_ack_report: str, optional
    :param isa_ack_status: The interchange acknowledgment status — either Accepted, Accepted with Errors, Partially Accepted, or Rejected, defaults to None
    :type isa_ack_status: str, optional
    :param isa_control: The number that uniquely identifies the interchange., defaults to None
    :type isa_control: str, optional
    :param operation_name: The name of the operation component that processed the record.
    :type operation_name: str
    :param outbound_validation_report: The outbound validation report. This report contains descriptions of errors present in the outbound interchange. If you did not select the outbound validation option in the sending trading partner, the value is N/A. The platform omits this field for an inbound interchange., defaults to None
    :type outbound_validation_report: str, optional
    :param outbound_validation_status: The outbound validation status — is either Success, Error-Interchange, Error-Transaction Set, or N/A. For an outbound interchange for which you did not select the outbound validation option in the sending trading partner, the value is N/A. The platform omits this field for an inbound interchange., defaults to None
    :type outbound_validation_status: str, optional
    :param receiver_id: For inbound interchanges, the Interchange Receiver ID, which identifies the receiver., defaults to None
    :type receiver_id: str, optional
    :param receiver_id_qualifier: For inbound interchanges, the Interchange ID Qualifier, which categorizes the Receiver ID., defaults to None
    :type receiver_id_qualifier: str, optional
    :param sender_id: For outbound interchanges, the Interchange Sender ID, which identifies the sender., defaults to None
    :type sender_id: str, optional
    :param sender_id_qualifier: For outbound interchanges, the Interchange ID Qualifier, which categorizes the Sender ID., defaults to None
    :type sender_id_qualifier: str, optional
    :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
    :type size: int, optional
    :param st_control: The transaction set control number., defaults to None
    :type st_control: str, optional
    :param standard: The Interchange Control standard., defaults to None
    :type standard: str, optional
    :param standard_id: Displays the same information as in the **Standard** column of the user interface., defaults to None
    :type standard_id: str, optional
    :param successful: Whether the record is a success or error., defaults to None
    :type successful: bool, optional
    :param test_indicator: Specifies whether the interchange is for testing or production., defaults to None
    :type test_indicator: str, optional
    :param transaction_set: The identifier code for the transaction set., defaults to None
    :type transaction_set: str, optional
    """

    def __init__(
        self,
        account: str,
        action_type: str,
        atom_id: str,
        connector_name: str,
        connector_type: str,
        custom_fields: CustomFields,
        date_processed: str,
        error_message: str,
        execution_id: str,
        id_: str,
        operation_name: str,
        ack_report: str = SENTINEL,
        ack_status: str = SENTINEL,
        agency_code: str = SENTINEL,
        app_receiver_id: str = SENTINEL,
        app_sender_id: str = SENTINEL,
        document_index: int = SENTINEL,
        functional_id: str = SENTINEL,
        gs_control: str = SENTINEL,
        gs_date: str = SENTINEL,
        gs_time: str = SENTINEL,
        gs_version: str = SENTINEL,
        isa_ack_report: str = SENTINEL,
        isa_ack_status: str = SENTINEL,
        isa_control: str = SENTINEL,
        outbound_validation_report: str = SENTINEL,
        outbound_validation_status: str = SENTINEL,
        receiver_id: str = SENTINEL,
        receiver_id_qualifier: str = SENTINEL,
        sender_id: str = SENTINEL,
        sender_id_qualifier: str = SENTINEL,
        size: int = SENTINEL,
        st_control: str = SENTINEL,
        standard: str = SENTINEL,
        standard_id: str = SENTINEL,
        successful: bool = SENTINEL,
        test_indicator: str = SENTINEL,
        transaction_set: str = SENTINEL,
        **kwargs,
    ):
        """X12ConnectorRecord

        :param account: The ID of the account in which this record ran.
        :type account: str
        :param ack_report: The acknowledgment report., defaults to None
        :type ack_report: str, optional
        :param ack_status: The acknowledgment status — either Accepted, Accepted with Errors, Partially Accepted, or Rejected., defaults to None
        :type ack_status: str, optional
        :param action_type: The type of action with which this record corresponds — Send for an outbound interchange, Get for an inbound interchange using the Disk, FTP, or SFTP communication method, or Listen for an inbound interchange using the AS2 or HTTP communication method.
        :type action_type: str
        :param agency_code: The Responsible Agency Code., defaults to None
        :type agency_code: str, optional
        :param app_receiver_id: For inbound interchanges, the Application Receiver’s Code, which identifies the receiver., defaults to None
        :type app_receiver_id: str, optional
        :param app_sender_id: For outbound interchanges, the Application Sender’s Code, which identifies the sender., defaults to None
        :type app_sender_id: str, optional
        :param atom_id: The ID of the Runtime that processed this record.
        :type atom_id: str
        :param connector_name: The value is Trading Partner for an X12 trading partner Send operation, or Start for an X12 trading partner Listen operation.
        :type connector_name: str
        :param connector_type: x12 is the connector type for any record.
        :type connector_type: str
        :param custom_fields: custom_fields
        :type custom_fields: CustomFields
        :param date_processed: The processing date and time for this record. The format is *yyyy-MM-dd'T'HH:mm:ss'Z'*, for example, 2016-01-31T15:32:00Z.
        :type date_processed: str
        :param document_index: The numerical index of this record in the run., defaults to None
        :type document_index: int, optional
        :param error_message: Any error message associated with this record.
        :type error_message: str
        :param execution_id: The ID of the run.
        :type execution_id: str
        :param functional_id: The identifier for the type of message in the functional group., defaults to None
        :type functional_id: str, optional
        :param gs_control: The group control number., defaults to None
        :type gs_control: str, optional
        :param gs_date: The preparation date of the interchange., defaults to None
        :type gs_date: str, optional
        :param gs_time: The preparation time of the interchange., defaults to None
        :type gs_time: str, optional
        :param gs_version: The Version, Release, and Industry identifier code., defaults to None
        :type gs_version: str, optional
        :param id_: The ID of this record.
        :type id_: str
        :param isa_ack_report: The interchange acknowledgment report, which contains descriptions of interchange segment validation errors., defaults to None
        :type isa_ack_report: str, optional
        :param isa_ack_status: The interchange acknowledgment status — either Accepted, Accepted with Errors, Partially Accepted, or Rejected, defaults to None
        :type isa_ack_status: str, optional
        :param isa_control: The number that uniquely identifies the interchange., defaults to None
        :type isa_control: str, optional
        :param operation_name: The name of the operation component that processed the record.
        :type operation_name: str
        :param outbound_validation_report: The outbound validation report. This report contains descriptions of errors present in the outbound interchange. If you did not select the outbound validation option in the sending trading partner, the value is N/A. The platform omits this field for an inbound interchange., defaults to None
        :type outbound_validation_report: str, optional
        :param outbound_validation_status: The outbound validation status — is either Success, Error-Interchange, Error-Transaction Set, or N/A. For an outbound interchange for which you did not select the outbound validation option in the sending trading partner, the value is N/A. The platform omits this field for an inbound interchange., defaults to None
        :type outbound_validation_status: str, optional
        :param receiver_id: For inbound interchanges, the Interchange Receiver ID, which identifies the receiver., defaults to None
        :type receiver_id: str, optional
        :param receiver_id_qualifier: For inbound interchanges, the Interchange ID Qualifier, which categorizes the Receiver ID., defaults to None
        :type receiver_id_qualifier: str, optional
        :param sender_id: For outbound interchanges, the Interchange Sender ID, which identifies the sender., defaults to None
        :type sender_id: str, optional
        :param sender_id_qualifier: For outbound interchanges, the Interchange ID Qualifier, which categorizes the Sender ID., defaults to None
        :type sender_id_qualifier: str, optional
        :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
        :type size: int, optional
        :param st_control: The transaction set control number., defaults to None
        :type st_control: str, optional
        :param standard: The Interchange Control standard., defaults to None
        :type standard: str, optional
        :param standard_id: Displays the same information as in the **Standard** column of the user interface., defaults to None
        :type standard_id: str, optional
        :param successful: Whether the record is a success or error., defaults to None
        :type successful: bool, optional
        :param test_indicator: Specifies whether the interchange is for testing or production., defaults to None
        :type test_indicator: str, optional
        :param transaction_set: The identifier code for the transaction set., defaults to None
        :type transaction_set: str, optional
        """
        self.account = account
        if ack_report is not SENTINEL:
            self.ack_report = ack_report
        if ack_status is not SENTINEL:
            self.ack_status = ack_status
        self.action_type = action_type
        if agency_code is not SENTINEL:
            self.agency_code = agency_code
        if app_receiver_id is not SENTINEL:
            self.app_receiver_id = app_receiver_id
        if app_sender_id is not SENTINEL:
            self.app_sender_id = app_sender_id
        self.atom_id = atom_id
        self.connector_name = connector_name
        self.connector_type = connector_type
        self.custom_fields = self._define_object(custom_fields, CustomFields)
        self.date_processed = date_processed
        if document_index is not SENTINEL:
            self.document_index = document_index
        self.error_message = error_message
        self.execution_id = execution_id
        if functional_id is not SENTINEL:
            self.functional_id = functional_id
        if gs_control is not SENTINEL:
            self.gs_control = gs_control
        if gs_date is not SENTINEL:
            self.gs_date = gs_date
        if gs_time is not SENTINEL:
            self.gs_time = gs_time
        if gs_version is not SENTINEL:
            self.gs_version = gs_version
        self.id_ = id_
        if isa_ack_report is not SENTINEL:
            self.isa_ack_report = isa_ack_report
        if isa_ack_status is not SENTINEL:
            self.isa_ack_status = isa_ack_status
        if isa_control is not SENTINEL:
            self.isa_control = isa_control
        self.operation_name = operation_name
        if outbound_validation_report is not SENTINEL:
            self.outbound_validation_report = outbound_validation_report
        if outbound_validation_status is not SENTINEL:
            self.outbound_validation_status = outbound_validation_status
        if receiver_id is not SENTINEL:
            self.receiver_id = receiver_id
        if receiver_id_qualifier is not SENTINEL:
            self.receiver_id_qualifier = receiver_id_qualifier
        if sender_id is not SENTINEL:
            self.sender_id = sender_id
        if sender_id_qualifier is not SENTINEL:
            self.sender_id_qualifier = sender_id_qualifier
        if size is not SENTINEL:
            self.size = size
        if st_control is not SENTINEL:
            self.st_control = st_control
        if standard is not SENTINEL:
            self.standard = standard
        if standard_id is not SENTINEL:
            self.standard_id = standard_id
        if successful is not SENTINEL:
            self.successful = successful
        if test_indicator is not SENTINEL:
            self.test_indicator = test_indicator
        if transaction_set is not SENTINEL:
            self.transaction_set = transaction_set
        self._kwargs = kwargs




import json
import re
from typing import Generator, Optional, Union
from requests import Response as RequestsResponse
from urllib.parse import parse_qs


class Response:
    """
    A simple HTTP response wrapper class using the requests library.

    :ivar int status: The status code of the HTTP response.
    :ivar dict headers: The headers of the HTTP response.
    :ivar str body: The body of the HTTP response.
    :var str chunk: The chunk of the HTTP response.
    """

    def __init__(
        self,
        response: RequestsResponse,
        chunk: Optional[str] = None,
        raw_chunk: Optional[bytes] = None,
    ) -> None:
        """
        Initializes a Response object.

        :param RequestsResponse response: The requests.Response object.
        """
        self.status = response.status_code
        self.headers = response.headers

        self.body = self._parse_response_body(
            content_type=response.headers.get("Content-Type", "").lower(),
            body=chunk if chunk else response.text,
            raw_body=raw_chunk if raw_chunk else response.content,
        )

    @staticmethod
    def from_chunk(
        response: RequestsResponse, raw_chunk: bytes
    ) -> Generator["Response", None, None]:
        """
        Create a Response object from a chunk of data.

        :param RequestsResponse response: The requests.Response object.
        :param bytes chunk: The chunk of data.
        :return: A Response object.
        :rtype: Response
        """
        content_type = response.headers.get("Content-Type", "").lower()
        chunk_str = raw_chunk.decode()
        if "text/event-stream" not in content_type:
            yield Response(response, chunk=chunk_str, raw_chunk=raw_chunk)
        else:
            for chunk_line in chunk_str.split("\n"):
                if "data: " in chunk_line:
                    yield Response(response, chunk=chunk_line, raw_chunk=raw_chunk)

    def __str__(self) -> str:
        """
        Return a string representation of the Response object.

        :return: A string representation of the Response object.
        :rtype: str
        """
        return (
            f"Response(status={self.status}, headers={self.headers}, body={self.body})"
        )

    def _parse_response_body(
        self, content_type: str, body: str, raw_body: bytes
    ) -> Union[str, dict, bytes]:
        """
        Extracts the response body from a given HTTP response.

        This method attempts to parse the response body based on its content type.
        If the content type is JSON, it tries to parse the body as JSON.
        If the content type is text or XML, it returns the raw text.
        If the content type is 'application/x-www-form-urlencoded', it parses the body as a query string.
        For all other content types, it returns the raw binary content.

        :param RequestsResponse response: The HTTP response received from a request.
        :return: The parsed response body.
        :rtype: str or dict or bytes
        """
        try:
            if re.search(r"application\/.*json", content_type):
                return json.loads(body)

            if "text/event-stream" in content_type and "data: " in body:
                json_body = body[6:]
                # Note: this assumes that the content of data is a valid JSON string
                return json.loads(json_body)

            if "text/" in content_type or content_type == "application/xml":
                return body

            if content_type == "application/x-www-form-urlencoded":
                parsed_response = parse_qs(body)
                return {k: v[0] for k, v in parsed_response.items()}

            return raw_body

        except json.JSONDecodeError:
            return raw_body




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .tradacoms_control_info import TradacomsControlInfo
from .tradacoms_options import TradacomsOptions


@JsonMap(
    {
        "tradacoms_control_info": "TradacomsControlInfo",
        "tradacoms_options": "TradacomsOptions",
    }
)
class TradacomsPartnerInfo(BaseModel):
    """TradacomsPartnerInfo

    :param tradacoms_control_info: tradacoms_control_info, defaults to None
    :type tradacoms_control_info: TradacomsControlInfo, optional
    :param tradacoms_options: tradacoms_options, defaults to None
    :type tradacoms_options: TradacomsOptions, optional
    """

    def __init__(
        self,
        tradacoms_control_info: TradacomsControlInfo = SENTINEL,
        tradacoms_options: TradacomsOptions = SENTINEL,
        **kwargs,
    ):
        """TradacomsPartnerInfo

        :param tradacoms_control_info: tradacoms_control_info, defaults to None
        :type tradacoms_control_info: TradacomsControlInfo, optional
        :param tradacoms_options: tradacoms_options, defaults to None
        :type tradacoms_options: TradacomsOptions, optional
        """
        if tradacoms_control_info is not SENTINEL:
            self.tradacoms_control_info = self._define_object(
                tradacoms_control_info, TradacomsControlInfo
            )
        if tradacoms_options is not SENTINEL:
            self.tradacoms_options = self._define_object(
                tradacoms_options, TradacomsOptions
            )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .folder_simple_expression import FolderSimpleExpression
from .folder_grouping_expression import FolderGroupingExpression


class FolderExpressionGuard(OneOfBaseModel):
    class_list = {
        "FolderSimpleExpression": FolderSimpleExpression,
        "FolderGroupingExpression": FolderGroupingExpression,
    }


FolderExpression = Union[FolderSimpleExpression, FolderGroupingExpression]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .schedule import Schedule


@JsonMap({"schedule": "Schedule", "max_retry": "maxRetry"})
class ScheduleRetry(BaseModel):
    """ScheduleRetry

    :param schedule: schedule, defaults to None
    :type schedule: List[Schedule], optional
    :param max_retry: (Retry schedules only) The maximum number of retries. The minimum valid value is 1; the maximum is 5., defaults to None
    :type max_retry: int, optional
    """

    def __init__(
        self, schedule: List[Schedule] = SENTINEL, max_retry: int = SENTINEL, **kwargs
    ):
        """ScheduleRetry

        :param schedule: schedule, defaults to None
        :type schedule: List[Schedule], optional
        :param max_retry: (Retry schedules only) The maximum number of retries. The minimum valid value is 1; the maximum is 5., defaults to None
        :type max_retry: int, optional
        """
        if schedule is not SENTINEL:
            self.schedule = self._define_list(schedule, Schedule)
        if max_retry is not SENTINEL:
            self.max_retry = max_retry
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .comp_diff_element import CompDiffElement


class CompDiffConfigComponentType(Enum):
    """An enumeration representing different categories.

    :cvar CERTIFICATE: "certificate"
    :vartype CERTIFICATE: str
    :cvar CONNECTORACTION: "connector-action"
    :vartype CONNECTORACTION: str
    :cvar CONNECTORSETTINGS: "connector-settings"
    :vartype CONNECTORSETTINGS: str
    :cvar CROSSREF: "crossref"
    :vartype CROSSREF: str
    :cvar DOCUMENTCACHE: "documentcache"
    :vartype DOCUMENTCACHE: str
    :cvar TRANSFORMMAP: "transform.map"
    :vartype TRANSFORMMAP: str
    :cvar TRANSFORMFUNCTION: "transform.function"
    :vartype TRANSFORMFUNCTION: str
    :cvar CERTIFICATEPGP: "certificate.pgp"
    :vartype CERTIFICATEPGP: str
    :cvar PROCESS: "process"
    :vartype PROCESS: str
    :cvar PROCESSPROPERTY: "processproperty"
    :vartype PROCESSPROPERTY: str
    :cvar PROFILEDB: "profile.db"
    :vartype PROFILEDB: str
    :cvar PROFILEEDI: "profile.edi"
    :vartype PROFILEEDI: str
    :cvar PROFILEFLATFILE: "profile.flatfile"
    :vartype PROFILEFLATFILE: str
    :cvar PROFILEXML: "profile.xml"
    :vartype PROFILEXML: str
    :cvar PROFILEJSON: "profile.json"
    :vartype PROFILEJSON: str
    :cvar QUEUE: "queue"
    :vartype QUEUE: str
    :cvar TRADINGPARTNER: "tradingpartner"
    :vartype TRADINGPARTNER: str
    :cvar TPGROUP: "tpgroup"
    :vartype TPGROUP: str
    :cvar TPORGANIZATION: "tporganization"
    :vartype TPORGANIZATION: str
    :cvar TPCOMMOPTIONS: "tpcommoptions"
    :vartype TPCOMMOPTIONS: str
    :cvar WEBSERVICE: "webservice"
    :vartype WEBSERVICE: str
    :cvar WEBSERVICEEXTERNAL: "webservice.external"
    :vartype WEBSERVICEEXTERNAL: str
    :cvar PROCESSROUTE: "processroute"
    :vartype PROCESSROUTE: str
    :cvar CUSTOMLIBRARY: "customlibrary"
    :vartype CUSTOMLIBRARY: str
    :cvar EDISTANDARD: "edistandard"
    :vartype EDISTANDARD: str
    :cvar FLOWSERVICE: "flowservice"
    :vartype FLOWSERVICE: str
    :cvar SCRIPTPROCESSING: "script.processing"
    :vartype SCRIPTPROCESSING: str
    :cvar SCRIPTMAPPING: "script.mapping"
    :vartype SCRIPTMAPPING: str
    :cvar XSLT: "xslt"
    :vartype XSLT: str
    """

    CERTIFICATE = "certificate"
    CONNECTORACTION = "connector-action"
    CONNECTORSETTINGS = "connector-settings"
    CROSSREF = "crossref"
    DOCUMENTCACHE = "documentcache"
    TRANSFORMMAP = "transform.map"
    TRANSFORMFUNCTION = "transform.function"
    CERTIFICATEPGP = "certificate.pgp"
    PROCESS = "process"
    PROCESSPROPERTY = "processproperty"
    PROFILEDB = "profile.db"
    PROFILEEDI = "profile.edi"
    PROFILEFLATFILE = "profile.flatfile"
    PROFILEXML = "profile.xml"
    PROFILEJSON = "profile.json"
    QUEUE = "queue"
    TRADINGPARTNER = "tradingpartner"
    TPGROUP = "tpgroup"
    TPORGANIZATION = "tporganization"
    TPCOMMOPTIONS = "tpcommoptions"
    WEBSERVICE = "webservice"
    WEBSERVICEEXTERNAL = "webservice.external"
    PROCESSROUTE = "processroute"
    CUSTOMLIBRARY = "customlibrary"
    EDISTANDARD = "edistandard"
    FLOWSERVICE = "flowservice"
    SCRIPTPROCESSING = "script.processing"
    SCRIPTMAPPING = "script.mapping"
    XSLT = "xslt"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, CompDiffConfigComponentType._member_map_.values())
        )


@JsonMap({"comp_diff_element": "CompDiffElement", "component_type": "componentType"})
class CompDiffConfig(BaseModel):
    """CompDiffConfig

    :param comp_diff_element: comp_diff_element, defaults to None
    :type comp_diff_element: List[CompDiffElement], optional
    :param component_type: The type of component that you want to compare., defaults to None
    :type component_type: CompDiffConfigComponentType, optional
    """

    def __init__(
        self,
        comp_diff_element: List[CompDiffElement] = SENTINEL,
        component_type: CompDiffConfigComponentType = SENTINEL,
        **kwargs,
    ):
        """CompDiffConfig

        :param comp_diff_element: comp_diff_element, defaults to None
        :type comp_diff_element: List[CompDiffElement], optional
        :param component_type: The type of component that you want to compare., defaults to None
        :type component_type: CompDiffConfigComponentType, optional
        """
        if comp_diff_element is not SENTINEL:
            self.comp_diff_element = self._define_list(
                comp_diff_element, CompDiffElement
            )
        if component_type is not SENTINEL:
            self.component_type = self._enum_matching(
                component_type, CompDiffConfigComponentType.list(), "component_type"
            )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .integration_pack_expression import (
    IntegrationPackExpression,
    IntegrationPackExpressionGuard,
)
from .integration_pack_simple_expression import IntegrationPackSimpleExpression
from .integration_pack_grouping_expression import IntegrationPackGroupingExpression


@JsonMap({})
class IntegrationPackQueryConfigQueryFilter(BaseModel):
    """IntegrationPackQueryConfigQueryFilter

    :param expression: expression
    :type expression: IntegrationPackExpression
    """

    def __init__(self, expression: IntegrationPackExpression, **kwargs):
        """IntegrationPackQueryConfigQueryFilter

        :param expression: expression
        :type expression: IntegrationPackExpression
        """
        self.expression = IntegrationPackExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class IntegrationPackQueryConfig(BaseModel):
    """IntegrationPackQueryConfig

    :param query_filter: query_filter
    :type query_filter: IntegrationPackQueryConfigQueryFilter
    """

    def __init__(self, query_filter: IntegrationPackQueryConfigQueryFilter, **kwargs):
        """IntegrationPackQueryConfig

        :param query_filter: query_filter
        :type query_filter: IntegrationPackQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, IntegrationPackQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "atom_id": "AtomId",
        "dlq": "DLQ",
        "queue_name": "QueueName",
        "subscriber_name": "SubscriberName",
    }
)
class ClearQueueRequest(BaseModel):
    """ClearQueueRequest

    :param atom_id: The unique ID assigned by the system to the container. \<br /\>The Runtime ID for Runtimes, Runtime clusters, and Runtime clouds is found in the user interface by navigating to **Manage** \\> **Runtime Management** and viewing the **Runtime Information** panel for the selected container.
    :type atom_id: str
    :param dlq: When set to *true*, it deletes messages from the regular queue only.When set to *false* \(default\), it deletes messages from the regular queue., defaults to None
    :type dlq: bool, optional
    :param queue_name: The name of the queue or topic. You can locate the queue or topic name by navigating to **Manage** \\> **Runtime Management** \\> **Queue Management panel** or by employing the ListQueues API action to retrieve the names of all queues on a given Runtime.
    :type queue_name: str
    :param subscriber_name: The subscriber name as it exists on the Runtime. You can find the subscriber name using the ListQueues API action or by looking up the Component ID of the process tied to the subscriber.   \>**Note:** The subscriber name does not necessarily equate to the Subscriber Name as is shown in Queue Management. If there is a process in Queue Management with the same name, use the Component ID of that process., defaults to None
    :type subscriber_name: str, optional
    """

    def __init__(
        self,
        atom_id: str,
        queue_name: str,
        dlq: bool = SENTINEL,
        subscriber_name: str = SENTINEL,
        **kwargs
    ):
        """ClearQueueRequest

        :param atom_id: The unique ID assigned by the system to the container. \<br /\>The Runtime ID for Runtimes, Runtime clusters, and Runtime clouds is found in the user interface by navigating to **Manage** \\> **Runtime Management** and viewing the **Runtime Information** panel for the selected container.
        :type atom_id: str
        :param dlq: When set to *true*, it deletes messages from the regular queue only.When set to *false* \(default\), it deletes messages from the regular queue., defaults to None
        :type dlq: bool, optional
        :param queue_name: The name of the queue or topic. You can locate the queue or topic name by navigating to **Manage** \\> **Runtime Management** \\> **Queue Management panel** or by employing the ListQueues API action to retrieve the names of all queues on a given Runtime.
        :type queue_name: str
        :param subscriber_name: The subscriber name as it exists on the Runtime. You can find the subscriber name using the ListQueues API action or by looking up the Component ID of the process tied to the subscriber.   \>**Note:** The subscriber name does not necessarily equate to the Subscriber Name as is shown in Queue Management. If there is a process in Queue Management with the same name, use the Component ID of that process., defaults to None
        :type subscriber_name: str, optional
        """
        self.atom_id = atom_id
        if dlq is not SENTINEL:
            self.dlq = dlq
        self.queue_name = queue_name
        if subscriber_name is not SENTINEL:
            self.subscriber_name = subscriber_name
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .custom_fields import CustomFields


@JsonMap(
    {
        "account": "account",
        "action_type": "actionType",
        "atom_id": "atomId",
        "connector_name": "connectorName",
        "connector_type": "connectorType",
        "custom_fields": "customFields",
        "date_processed": "dateProcessed",
        "document_index": "documentIndex",
        "error_message": "errorMessage",
        "execution_id": "executionId",
        "id_": "id",
        "initiator_ssidcode": "initiator_ssidcode",
        "nareas": "nareas",
        "nareast": "nareast",
        "objecttype": "objecttype",
        "operation_name": "operationName",
        "responder_ssidcode": "responder_ssidcode",
        "sfidciph": "sfidciph",
        "sfidcomp": "sfidcomp",
        "sfiddate": "sfiddate",
        "sfiddesc": "sfiddesc",
        "sfiddest": "sfiddest",
        "sfiddsn": "sfiddsn",
        "sfidenv": "sfidenv",
        "sfidorig": "sfidorig",
        "sfidosiz": "sfidosiz",
        "sfidsec": "sfidsec",
        "sfidsign": "sfidsign",
        "sfidtime": "sfidtime",
        "size": "size",
        "status": "status",
        "successful": "successful",
        "ticker": "ticker",
    }
)
class Oftp2ConnectorRecord(BaseModel):
    """Oftp2ConnectorRecord

    :param account: The ID of the account from which you ran this record.
    :type account: str
    :param action_type: The type of action with which this record corresponds - Send, Get or Listen.
    :type action_type: str
    :param atom_id: The ID of the Runtime that processed this record.
    :type atom_id: str
    :param connector_name: For the OFTP2 Client, the value is oftp Connector and for the OFTP2 Server, the value is oftpserver Connector.
    :type connector_name: str
    :param connector_type: The type of connector to which this record corresponds- oftp for OFTP2 Client or oftpserver for OFTP2 Server.
    :type connector_type: str
    :param custom_fields: custom_fields
    :type custom_fields: CustomFields
    :param date_processed: The processing date and time of this record. The format is yyyy-MM-dd'T'HH:mm:ss'Z', for example, 2013-08-08T15:32:00Z.
    :type date_processed: str
    :param document_index: The numerical index of this record in the run., defaults to None
    :type document_index: int, optional
    :param error_message: error_message
    :type error_message: str
    :param execution_id: The ID of the execution run.
    :type execution_id: str
    :param id_: The ID of this record.
    :type id_: str
    :param initiator_ssidcode: The Initiator's Identification Code., defaults to None
    :type initiator_ssidcode: str, optional
    :param nareas: Only displays if the transmission failed. The reason numeric code for the failure., defaults to None
    :type nareas: str, optional
    :param nareast: Only displays if the transmission failed. The reason text for the failure., defaults to None
    :type nareast: str, optional
    :param objecttype: objecttype, defaults to None
    :type objecttype: str, optional
    :param operation_name: The name of the operation component that processed this record.
    :type operation_name: str
    :param responder_ssidcode: The Responder's Identification Code., defaults to None
    :type responder_ssidcode: str, optional
    :param sfidciph: Indicates which cipher suite was used to sign or encrypt the file. The cipher suite in this value should also be used when a signed EERP or NERP is requested.\<br /\>-`00` - No security services\<br /\>-`01` - 3DES_EDE_CBC_3KEY\<br /\> -`02` - AES_256_CBC, defaults to None
    :type sfidciph: str, optional
    :param sfidcomp: Indicates whether an algorithm was used to compress the file.\<br /\>-`0` - No compression\<br /\>-`1` - Compressed with an algorithm., defaults to None
    :type sfidcomp: str, optional
    :param sfiddate: The date when the virtual file was created. The format is yyyy-MM-dd, for example 2023-06-07., defaults to None
    :type sfiddate: str, optional
    :param sfiddesc: The description of the virtual file., defaults to None
    :type sfiddesc: str, optional
    :param sfiddest: The destination Odette ID for the virtual file., defaults to None
    :type sfiddest: str, optional
    :param sfiddsn: The dataset name of the virtual file being transferred., defaults to None
    :type sfiddsn: str, optional
    :param sfidenv: The enveloping format used in the file.\<br /\>-`0` - No enveloped\<br /\>-`1` - File is enveloped using CMS, defaults to None
    :type sfidenv: str, optional
    :param sfidorig: The originator of the virtual file., defaults to None
    :type sfidorig: str, optional
    :param sfidosiz: The size of the original file., defaults to None
    :type sfidosiz: str, optional
    :param sfidsec: Indicates whether the file has been signed or encrypted before transmission. The following values are possible:\<br /\>-`00` - No security services\<br /\>-`01` - Encrypted\<br /\>-`02` - Signed\<br /\>-`03` - Encrypted and signed, defaults to None
    :type sfidsec: str, optional
    :param sfidsign: Whether the EERP returned for the file must be signed.\<br /\>-`Y` - the EERP must be signed\<br /\>-`N` - The EERP must not be signed, defaults to None
    :type sfidsign: str, optional
    :param sfidtime: The time when the virtual file was created. The format is HH:mm:ss.SSSX, where X is the ticker, for example 10:06:46.2389., defaults to None
    :type sfidtime: str, optional
    :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
    :type size: int, optional
    :param status: Whether the file transmission is pending an acknowledgment, acknowledged as a success or an error., defaults to None
    :type status: str, optional
    :param successful: Whether the record is a success or error., defaults to None
    :type successful: bool, optional
    :param ticker: ticker, defaults to None
    :type ticker: str, optional
    """

    def __init__(
        self,
        account: str,
        action_type: str,
        atom_id: str,
        connector_name: str,
        connector_type: str,
        custom_fields: CustomFields,
        date_processed: str,
        error_message: str,
        execution_id: str,
        id_: str,
        operation_name: str,
        document_index: int = SENTINEL,
        initiator_ssidcode: str = SENTINEL,
        nareas: str = SENTINEL,
        nareast: str = SENTINEL,
        objecttype: str = SENTINEL,
        responder_ssidcode: str = SENTINEL,
        sfidciph: str = SENTINEL,
        sfidcomp: str = SENTINEL,
        sfiddate: str = SENTINEL,
        sfiddesc: str = SENTINEL,
        sfiddest: str = SENTINEL,
        sfiddsn: str = SENTINEL,
        sfidenv: str = SENTINEL,
        sfidorig: str = SENTINEL,
        sfidosiz: str = SENTINEL,
        sfidsec: str = SENTINEL,
        sfidsign: str = SENTINEL,
        sfidtime: str = SENTINEL,
        size: int = SENTINEL,
        status: str = SENTINEL,
        successful: bool = SENTINEL,
        ticker: str = SENTINEL,
        **kwargs,
    ):
        """Oftp2ConnectorRecord

        :param account: The ID of the account from which you ran this record.
        :type account: str
        :param action_type: The type of action with which this record corresponds - Send, Get or Listen.
        :type action_type: str
        :param atom_id: The ID of the Runtime that processed this record.
        :type atom_id: str
        :param connector_name: For the OFTP2 Client, the value is oftp Connector and for the OFTP2 Server, the value is oftpserver Connector.
        :type connector_name: str
        :param connector_type: The type of connector to which this record corresponds- oftp for OFTP2 Client or oftpserver for OFTP2 Server.
        :type connector_type: str
        :param custom_fields: custom_fields
        :type custom_fields: CustomFields
        :param date_processed: The processing date and time of this record. The format is yyyy-MM-dd'T'HH:mm:ss'Z', for example, 2013-08-08T15:32:00Z.
        :type date_processed: str
        :param document_index: The numerical index of this record in the run., defaults to None
        :type document_index: int, optional
        :param error_message: error_message
        :type error_message: str
        :param execution_id: The ID of the execution run.
        :type execution_id: str
        :param id_: The ID of this record.
        :type id_: str
        :param initiator_ssidcode: The Initiator's Identification Code., defaults to None
        :type initiator_ssidcode: str, optional
        :param nareas: Only displays if the transmission failed. The reason numeric code for the failure., defaults to None
        :type nareas: str, optional
        :param nareast: Only displays if the transmission failed. The reason text for the failure., defaults to None
        :type nareast: str, optional
        :param objecttype: objecttype, defaults to None
        :type objecttype: str, optional
        :param operation_name: The name of the operation component that processed this record.
        :type operation_name: str
        :param responder_ssidcode: The Responder's Identification Code., defaults to None
        :type responder_ssidcode: str, optional
        :param sfidciph: Indicates which cipher suite was used to sign or encrypt the file. The cipher suite in this value should also be used when a signed EERP or NERP is requested.\<br /\>-`00` - No security services\<br /\>-`01` - 3DES_EDE_CBC_3KEY\<br /\> -`02` - AES_256_CBC, defaults to None
        :type sfidciph: str, optional
        :param sfidcomp: Indicates whether an algorithm was used to compress the file.\<br /\>-`0` - No compression\<br /\>-`1` - Compressed with an algorithm., defaults to None
        :type sfidcomp: str, optional
        :param sfiddate: The date when the virtual file was created. The format is yyyy-MM-dd, for example 2023-06-07., defaults to None
        :type sfiddate: str, optional
        :param sfiddesc: The description of the virtual file., defaults to None
        :type sfiddesc: str, optional
        :param sfiddest: The destination Odette ID for the virtual file., defaults to None
        :type sfiddest: str, optional
        :param sfiddsn: The dataset name of the virtual file being transferred., defaults to None
        :type sfiddsn: str, optional
        :param sfidenv: The enveloping format used in the file.\<br /\>-`0` - No enveloped\<br /\>-`1` - File is enveloped using CMS, defaults to None
        :type sfidenv: str, optional
        :param sfidorig: The originator of the virtual file., defaults to None
        :type sfidorig: str, optional
        :param sfidosiz: The size of the original file., defaults to None
        :type sfidosiz: str, optional
        :param sfidsec: Indicates whether the file has been signed or encrypted before transmission. The following values are possible:\<br /\>-`00` - No security services\<br /\>-`01` - Encrypted\<br /\>-`02` - Signed\<br /\>-`03` - Encrypted and signed, defaults to None
        :type sfidsec: str, optional
        :param sfidsign: Whether the EERP returned for the file must be signed.\<br /\>-`Y` - the EERP must be signed\<br /\>-`N` - The EERP must not be signed, defaults to None
        :type sfidsign: str, optional
        :param sfidtime: The time when the virtual file was created. The format is HH:mm:ss.SSSX, where X is the ticker, for example 10:06:46.2389., defaults to None
        :type sfidtime: str, optional
        :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
        :type size: int, optional
        :param status: Whether the file transmission is pending an acknowledgment, acknowledged as a success or an error., defaults to None
        :type status: str, optional
        :param successful: Whether the record is a success or error., defaults to None
        :type successful: bool, optional
        :param ticker: ticker, defaults to None
        :type ticker: str, optional
        """
        self.account = account
        self.action_type = action_type
        self.atom_id = atom_id
        self.connector_name = connector_name
        self.connector_type = connector_type
        self.custom_fields = self._define_object(custom_fields, CustomFields)
        self.date_processed = date_processed
        if document_index is not SENTINEL:
            self.document_index = document_index
        self.error_message = error_message
        self.execution_id = execution_id
        self.id_ = id_
        if initiator_ssidcode is not SENTINEL:
            self.initiator_ssidcode = initiator_ssidcode
        if nareas is not SENTINEL:
            self.nareas = nareas
        if nareast is not SENTINEL:
            self.nareast = nareast
        if objecttype is not SENTINEL:
            self.objecttype = objecttype
        self.operation_name = operation_name
        if responder_ssidcode is not SENTINEL:
            self.responder_ssidcode = responder_ssidcode
        if sfidciph is not SENTINEL:
            self.sfidciph = sfidciph
        if sfidcomp is not SENTINEL:
            self.sfidcomp = sfidcomp
        if sfiddate is not SENTINEL:
            self.sfiddate = sfiddate
        if sfiddesc is not SENTINEL:
            self.sfiddesc = sfiddesc
        if sfiddest is not SENTINEL:
            self.sfiddest = sfiddest
        if sfiddsn is not SENTINEL:
            self.sfiddsn = sfiddsn
        if sfidenv is not SENTINEL:
            self.sfidenv = sfidenv
        if sfidorig is not SENTINEL:
            self.sfidorig = sfidorig
        if sfidosiz is not SENTINEL:
            self.sfidosiz = sfidosiz
        if sfidsec is not SENTINEL:
            self.sfidsec = sfidsec
        if sfidsign is not SENTINEL:
            self.sfidsign = sfidsign
        if sfidtime is not SENTINEL:
            self.sfidtime = sfidtime
        if size is not SENTINEL:
            self.size = size
        if status is not SENTINEL:
            self.status = status
        if successful is not SENTINEL:
            self.successful = successful
        if ticker is not SENTINEL:
            self.ticker = ticker
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .cloud_atom import CloudAtom


@JsonMap({"atom": "Atom", "id_": "id"})
class Cloud(BaseModel):
    """Cloud

    :param atom: atom, defaults to None
    :type atom: List[CloudAtom], optional
    :param id_: A unique ID assigned by the system to the Runtime cloud., defaults to None
    :type id_: str, optional
    :param name: The name of the Runtime cloud., defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        atom: List[CloudAtom] = SENTINEL,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        **kwargs,
    ):
        """Cloud

        :param atom: atom, defaults to None
        :type atom: List[CloudAtom], optional
        :param id_: A unique ID assigned by the system to the Runtime cloud., defaults to None
        :type id_: str, optional
        :param name: The name of the Runtime cloud., defaults to None
        :type name: str, optional
        """
        if atom is not SENTINEL:
            self.atom = self._define_list(atom, CloudAtom)
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment_map_extensions_summary import EnvironmentMapExtensionsSummaryService
from ...models import (
    EnvironmentMapExtensionsSummaryQueryResponse,
    EnvironmentMapExtensionsSummaryQueryConfig,
)


class EnvironmentMapExtensionsSummaryServiceAsync(
    EnvironmentMapExtensionsSummaryService
):
    """
    Async Wrapper for EnvironmentMapExtensionsSummaryServiceAsync
    """

    def query_environment_map_extensions_summary(
        self, request_body: EnvironmentMapExtensionsSummaryQueryConfig = None
    ) -> Awaitable[Union[EnvironmentMapExtensionsSummaryQueryResponse, str]]:
        return to_async(super().query_environment_map_extensions_summary)(request_body)

    def query_more_environment_map_extensions_summary(
        self, request_body: str
    ) -> Awaitable[Union[EnvironmentMapExtensionsSummaryQueryResponse, str]]:
        return to_async(super().query_more_environment_map_extensions_summary)(
            request_body
        )




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..refresh_secrets_manager import RefreshSecretsManagerService
from ...models import SecretsManagerRefreshResponse, SecretsManagerRefreshRequest


class RefreshSecretsManagerServiceAsync(RefreshSecretsManagerService):
    """
    Async Wrapper for RefreshSecretsManagerServiceAsync
    """

    def refresh_secrets_manager(
        self, request_body: SecretsManagerRefreshRequest = None
    ) -> Awaitable[Union[SecretsManagerRefreshResponse, str]]:
        return to_async(super().refresh_secrets_manager)(request_body)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..process_atom_attachment import ProcessAtomAttachmentService
from ...models import (
    ProcessAtomAttachment,
    ProcessAtomAttachmentQueryResponse,
    ProcessAtomAttachmentQueryConfig,
)


class ProcessAtomAttachmentServiceAsync(ProcessAtomAttachmentService):
    """
    Async Wrapper for ProcessAtomAttachmentServiceAsync
    """

    def create_process_atom_attachment(
        self, request_body: ProcessAtomAttachment = None
    ) -> Awaitable[Union[ProcessAtomAttachment, str]]:
        return to_async(super().create_process_atom_attachment)(request_body)

    def query_process_atom_attachment(
        self, request_body: ProcessAtomAttachmentQueryConfig = None
    ) -> Awaitable[Union[ProcessAtomAttachmentQueryResponse, str]]:
        return to_async(super().query_process_atom_attachment)(request_body)

    def query_more_process_atom_attachment(
        self, request_body: str
    ) -> Awaitable[Union[ProcessAtomAttachmentQueryResponse, str]]:
        return to_async(super().query_more_process_atom_attachment)(request_body)

    def delete_process_atom_attachment(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_process_atom_attachment)(id_)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..atom_log import AtomLogService
from ...models import LogDownload, AtomLog


class AtomLogServiceAsync(AtomLogService):
    """
    Async Wrapper for AtomLogServiceAsync
    """

    def create_atom_log(
        self, request_body: AtomLog = None
    ) -> Awaitable[Union[LogDownload, str]]:
        return to_async(super().create_atom_log)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"fixed_length": "fixedLength"})
class MapExtensionsStringConcat(BaseModel):
    """MapExtensionsStringConcat

    :param delimiter: delimiter, defaults to None
    :type delimiter: str, optional
    :param fixed_length: fixed_length, defaults to None
    :type fixed_length: int, optional
    """

    def __init__(
        self, delimiter: str = SENTINEL, fixed_length: int = SENTINEL, **kwargs
    ):
        """MapExtensionsStringConcat

        :param delimiter: delimiter, defaults to None
        :type delimiter: str, optional
        :param fixed_length: fixed_length, defaults to None
        :type fixed_length: int, optional
        """
        if delimiter is not SENTINEL:
            self.delimiter = delimiter
        if fixed_length is not SENTINEL:
            self.fixed_length = fixed_length
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    EnvironmentAtomAttachment,
    EnvironmentAtomAttachmentQueryConfig,
    EnvironmentAtomAttachmentQueryResponse,
)


class EnvironmentAtomAttachmentService(BaseService):

    @cast_models
    def create_environment_atom_attachment(
        self, request_body: EnvironmentAtomAttachment = None
    ) -> Union[EnvironmentAtomAttachment, str]:
        """Attaches a Runtime having the specified ID to the environment having the specified ID. Attaching an already attached Runtime moves the Runtime to the environment specified in the request.

         >**Note:** For accounts with Basic environment support, you can attach a single Runtime to each environment. For accounts with Unlimited environment support, you can attach have an unlimited number of Runtimes attached in each environment.

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentAtomAttachment, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentAtomAttachment, str]
        """

        Validator(EnvironmentAtomAttachment).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentAtomAttachment",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentAtomAttachment._unmap(response)
        if content == "application/xml":
            return EnvironmentAtomAttachment._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_environment_atom_attachment(
        self, request_body: EnvironmentAtomAttachmentQueryConfig = None
    ) -> Union[EnvironmentAtomAttachmentQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentAtomAttachmentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentAtomAttachmentQueryResponse, str]
        """

        Validator(EnvironmentAtomAttachmentQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentAtomAttachment/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentAtomAttachmentQueryResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentAtomAttachmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_environment_atom_attachment(
        self, request_body: str
    ) -> Union[EnvironmentAtomAttachmentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentAtomAttachmentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentAtomAttachment/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentAtomAttachmentQueryResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentAtomAttachmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_environment_atom_attachment(self, id_: str) -> None:
        """Detaches a Runtime from an environment where the attachment is specified by the conceptual Environment Atom Attachment object ID. This ID is returned by the CREATE operation that originated the attachment and can also be obtained from a QUERY operation. If you successfully detach the Runtime from the environment, the response is  `<true/>`.

        :param id_: The object’s conceptual ID, which is synthesized from the Runtime and environment IDs.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentAtomAttachment/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class ConnectorVersion(BaseModel):
    """Each pair of `name` and `version` fields represents a connector listed on the **Runtime Management page > Runtime & Connector Versions** tab.

    :param name: The connector name used by the Runtime, Runtime cluster, or Runtime cloud., defaults to None
    :type name: str, optional
    :param version: The connector version used by the Runtime, Runtime cluster, or Runtime cloud., defaults to None
    :type version: str, optional
    """

    def __init__(self, name: str = SENTINEL, version: str = SENTINEL, **kwargs):
        """Each pair of `name` and `version` fields represents a connector listed on the **Runtime Management page > Runtime & Connector Versions** tab.

        :param name: The connector name used by the Runtime, Runtime cluster, or Runtime cloud., defaults to None
        :type name: str, optional
        :param version: The connector version used by the Runtime, Runtime cluster, or Runtime cloud., defaults to None
        :type version: str, optional
        """
        if name is not SENTINEL:
            self.name = name
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .custom_fields import CustomFields


@JsonMap(
    {
        "account": "account",
        "ack_report": "ackReport",
        "ack_requested": "ackRequested",
        "ack_status": "ackStatus",
        "action_type": "actionType",
        "atom_id": "atomId",
        "connector_name": "connectorName",
        "connector_type": "connectorType",
        "controlling_agency": "controllingAgency",
        "custom_fields": "customFields",
        "date_processed": "dateProcessed",
        "document_index": "documentIndex",
        "error_message": "errorMessage",
        "execution_id": "executionId",
        "id_": "id",
        "interchange_control_reference": "interchangeControlReference",
        "interchange_date": "interchangeDate",
        "interchange_time": "interchangeTime",
        "message_reference_number": "messageReferenceNumber",
        "message_type": "messageType",
        "operation_name": "operationName",
        "outbound_validation_report": "outboundValidationReport",
        "outbound_validation_status": "outboundValidationStatus",
        "receiver_id": "receiverID",
        "release": "release",
        "sender_id": "senderID",
        "size": "size",
        "successful": "successful",
        "version": "version",
    }
)
class OdetteConnectorRecord(BaseModel):
    """OdetteConnectorRecord

    :param account: account
    :type account: str
    :param ack_report: ack_report, defaults to None
    :type ack_report: str, optional
    :param ack_requested: ack_requested, defaults to None
    :type ack_requested: str, optional
    :param ack_status: ack_status, defaults to None
    :type ack_status: str, optional
    :param action_type: action_type
    :type action_type: str
    :param atom_id: atom_id
    :type atom_id: str
    :param connector_name: connector_name
    :type connector_name: str
    :param connector_type: connector_type
    :type connector_type: str
    :param controlling_agency: controlling_agency, defaults to None
    :type controlling_agency: str, optional
    :param custom_fields: custom_fields
    :type custom_fields: CustomFields
    :param date_processed: date_processed
    :type date_processed: str
    :param document_index: document_index, defaults to None
    :type document_index: int, optional
    :param error_message: error_message
    :type error_message: str
    :param execution_id: execution_id
    :type execution_id: str
    :param id_: id_
    :type id_: str
    :param interchange_control_reference: interchange_control_reference, defaults to None
    :type interchange_control_reference: str, optional
    :param interchange_date: interchange_date, defaults to None
    :type interchange_date: str, optional
    :param interchange_time: interchange_time, defaults to None
    :type interchange_time: str, optional
    :param message_reference_number: message_reference_number, defaults to None
    :type message_reference_number: str, optional
    :param message_type: message_type, defaults to None
    :type message_type: str, optional
    :param operation_name: operation_name
    :type operation_name: str
    :param outbound_validation_report: outbound_validation_report, defaults to None
    :type outbound_validation_report: str, optional
    :param outbound_validation_status: outbound_validation_status, defaults to None
    :type outbound_validation_status: str, optional
    :param receiver_id: receiver_id, defaults to None
    :type receiver_id: str, optional
    :param release: release, defaults to None
    :type release: str, optional
    :param sender_id: sender_id, defaults to None
    :type sender_id: str, optional
    :param size: size, defaults to None
    :type size: int, optional
    :param successful: successful, defaults to None
    :type successful: bool, optional
    :param version: version, defaults to None
    :type version: str, optional
    """

    def __init__(
        self,
        account: str,
        action_type: str,
        atom_id: str,
        connector_name: str,
        connector_type: str,
        custom_fields: CustomFields,
        date_processed: str,
        error_message: str,
        execution_id: str,
        id_: str,
        operation_name: str,
        ack_report: str = SENTINEL,
        ack_requested: str = SENTINEL,
        ack_status: str = SENTINEL,
        controlling_agency: str = SENTINEL,
        document_index: int = SENTINEL,
        interchange_control_reference: str = SENTINEL,
        interchange_date: str = SENTINEL,
        interchange_time: str = SENTINEL,
        message_reference_number: str = SENTINEL,
        message_type: str = SENTINEL,
        outbound_validation_report: str = SENTINEL,
        outbound_validation_status: str = SENTINEL,
        receiver_id: str = SENTINEL,
        release: str = SENTINEL,
        sender_id: str = SENTINEL,
        size: int = SENTINEL,
        successful: bool = SENTINEL,
        version: str = SENTINEL,
        **kwargs,
    ):
        """OdetteConnectorRecord

        :param account: account
        :type account: str
        :param ack_report: ack_report, defaults to None
        :type ack_report: str, optional
        :param ack_requested: ack_requested, defaults to None
        :type ack_requested: str, optional
        :param ack_status: ack_status, defaults to None
        :type ack_status: str, optional
        :param action_type: action_type
        :type action_type: str
        :param atom_id: atom_id
        :type atom_id: str
        :param connector_name: connector_name
        :type connector_name: str
        :param connector_type: connector_type
        :type connector_type: str
        :param controlling_agency: controlling_agency, defaults to None
        :type controlling_agency: str, optional
        :param custom_fields: custom_fields
        :type custom_fields: CustomFields
        :param date_processed: date_processed
        :type date_processed: str
        :param document_index: document_index, defaults to None
        :type document_index: int, optional
        :param error_message: error_message
        :type error_message: str
        :param execution_id: execution_id
        :type execution_id: str
        :param id_: id_
        :type id_: str
        :param interchange_control_reference: interchange_control_reference, defaults to None
        :type interchange_control_reference: str, optional
        :param interchange_date: interchange_date, defaults to None
        :type interchange_date: str, optional
        :param interchange_time: interchange_time, defaults to None
        :type interchange_time: str, optional
        :param message_reference_number: message_reference_number, defaults to None
        :type message_reference_number: str, optional
        :param message_type: message_type, defaults to None
        :type message_type: str, optional
        :param operation_name: operation_name
        :type operation_name: str
        :param outbound_validation_report: outbound_validation_report, defaults to None
        :type outbound_validation_report: str, optional
        :param outbound_validation_status: outbound_validation_status, defaults to None
        :type outbound_validation_status: str, optional
        :param receiver_id: receiver_id, defaults to None
        :type receiver_id: str, optional
        :param release: release, defaults to None
        :type release: str, optional
        :param sender_id: sender_id, defaults to None
        :type sender_id: str, optional
        :param size: size, defaults to None
        :type size: int, optional
        :param successful: successful, defaults to None
        :type successful: bool, optional
        :param version: version, defaults to None
        :type version: str, optional
        """
        self.account = account
        if ack_report is not SENTINEL:
            self.ack_report = ack_report
        if ack_requested is not SENTINEL:
            self.ack_requested = ack_requested
        if ack_status is not SENTINEL:
            self.ack_status = ack_status
        self.action_type = action_type
        self.atom_id = atom_id
        self.connector_name = connector_name
        self.connector_type = connector_type
        if controlling_agency is not SENTINEL:
            self.controlling_agency = controlling_agency
        self.custom_fields = self._define_object(custom_fields, CustomFields)
        self.date_processed = date_processed
        if document_index is not SENTINEL:
            self.document_index = document_index
        self.error_message = error_message
        self.execution_id = execution_id
        self.id_ = id_
        if interchange_control_reference is not SENTINEL:
            self.interchange_control_reference = interchange_control_reference
        if interchange_date is not SENTINEL:
            self.interchange_date = interchange_date
        if interchange_time is not SENTINEL:
            self.interchange_time = interchange_time
        if message_reference_number is not SENTINEL:
            self.message_reference_number = message_reference_number
        if message_type is not SENTINEL:
            self.message_type = message_type
        self.operation_name = operation_name
        if outbound_validation_report is not SENTINEL:
            self.outbound_validation_report = outbound_validation_report
        if outbound_validation_status is not SENTINEL:
            self.outbound_validation_status = outbound_validation_status
        if receiver_id is not SENTINEL:
            self.receiver_id = receiver_id
        if release is not SENTINEL:
            self.release = release
        if sender_id is not SENTINEL:
            self.sender_id = sender_id
        if size is not SENTINEL:
            self.size = size
        if successful is not SENTINEL:
            self.successful = successful
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .edifact_partner_info import EdifactPartnerInfo
from .hl7_partner_info import Hl7PartnerInfo
from .odette_partner_info import OdettePartnerInfo
from .rosetta_net_partner_info import RosettaNetPartnerInfo
from .tradacoms_partner_info import TradacomsPartnerInfo
from .x12_partner_info import X12PartnerInfo


@JsonMap(
    {
        "custom_partner_info": "CustomPartnerInfo",
        "edifact_partner_info": "EdifactPartnerInfo",
        "hl7_partner_info": "HL7PartnerInfo",
        "odette_partner_info": "OdettePartnerInfo",
        "rosetta_net_partner_info": "RosettaNetPartnerInfo",
        "tradacoms_partner_info": "TradacomsPartnerInfo",
        "x12_partner_info": "X12PartnerInfo",
    }
)
class PartnerInfo(BaseModel):
    """PartnerInfo

    :param custom_partner_info: custom_partner_info, defaults to None
    :type custom_partner_info: dict, optional
    :param edifact_partner_info: edifact_partner_info, defaults to None
    :type edifact_partner_info: EdifactPartnerInfo, optional
    :param hl7_partner_info: hl7_partner_info, defaults to None
    :type hl7_partner_info: Hl7PartnerInfo, optional
    :param odette_partner_info: odette_partner_info, defaults to None
    :type odette_partner_info: OdettePartnerInfo, optional
    :param rosetta_net_partner_info: rosetta_net_partner_info, defaults to None
    :type rosetta_net_partner_info: RosettaNetPartnerInfo, optional
    :param tradacoms_partner_info: tradacoms_partner_info, defaults to None
    :type tradacoms_partner_info: TradacomsPartnerInfo, optional
    :param x12_partner_info: x12_partner_info, defaults to None
    :type x12_partner_info: X12PartnerInfo, optional
    """

    def __init__(
        self,
        custom_partner_info: dict = SENTINEL,
        edifact_partner_info: EdifactPartnerInfo = SENTINEL,
        hl7_partner_info: Hl7PartnerInfo = SENTINEL,
        odette_partner_info: OdettePartnerInfo = SENTINEL,
        rosetta_net_partner_info: RosettaNetPartnerInfo = SENTINEL,
        tradacoms_partner_info: TradacomsPartnerInfo = SENTINEL,
        x12_partner_info: X12PartnerInfo = SENTINEL,
        **kwargs,
    ):
        """PartnerInfo

        :param custom_partner_info: custom_partner_info, defaults to None
        :type custom_partner_info: dict, optional
        :param edifact_partner_info: edifact_partner_info, defaults to None
        :type edifact_partner_info: EdifactPartnerInfo, optional
        :param hl7_partner_info: hl7_partner_info, defaults to None
        :type hl7_partner_info: Hl7PartnerInfo, optional
        :param odette_partner_info: odette_partner_info, defaults to None
        :type odette_partner_info: OdettePartnerInfo, optional
        :param rosetta_net_partner_info: rosetta_net_partner_info, defaults to None
        :type rosetta_net_partner_info: RosettaNetPartnerInfo, optional
        :param tradacoms_partner_info: tradacoms_partner_info, defaults to None
        :type tradacoms_partner_info: TradacomsPartnerInfo, optional
        :param x12_partner_info: x12_partner_info, defaults to None
        :type x12_partner_info: X12PartnerInfo, optional
        """
        if custom_partner_info is not SENTINEL:
            self.custom_partner_info = custom_partner_info
        if edifact_partner_info is not SENTINEL:
            self.edifact_partner_info = self._define_object(
                edifact_partner_info, EdifactPartnerInfo
            )
        if hl7_partner_info is not SENTINEL:
            self.hl7_partner_info = self._define_object(
                hl7_partner_info, Hl7PartnerInfo
            )
        if odette_partner_info is not SENTINEL:
            self.odette_partner_info = self._define_object(
                odette_partner_info, OdettePartnerInfo
            )
        if rosetta_net_partner_info is not SENTINEL:
            self.rosetta_net_partner_info = self._define_object(
                rosetta_net_partner_info, RosettaNetPartnerInfo
            )
        if tradacoms_partner_info is not SENTINEL:
            self.tradacoms_partner_info = self._define_object(
                tradacoms_partner_info, TradacomsPartnerInfo
            )
        if x12_partner_info is not SENTINEL:
            self.x12_partner_info = self._define_object(
                x12_partner_info, X12PartnerInfo
            )
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .as2_send_settings import As2SendSettings
from .as2_receive_options import As2ReceiveOptions
from .as2_send_options import As2SendOptions
from .shared_communication_channel import SharedCommunicationChannel


class As2CommunicationOptionsCommunicationSetting(Enum):
    """An enumeration representing different categories.

    :cvar DEFAULT: "default"
    :vartype DEFAULT: str
    :cvar CUSTOM: "custom"
    :vartype CUSTOM: str
    :cvar COMPONENT: "component"
    :vartype COMPONENT: str
    """

    DEFAULT = "default"
    CUSTOM = "custom"
    COMPONENT = "component"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                As2CommunicationOptionsCommunicationSetting._member_map_.values(),
            )
        )


@JsonMap(
    {
        "as2_default_partner_settings": "AS2DefaultPartnerSettings",
        "as2_receive_options": "AS2ReceiveOptions",
        "as2_send_options": "AS2SendOptions",
        "as2_send_settings": "AS2SendSettings",
        "communication_setting": "CommunicationSetting",
        "shared_communication_channel": "SharedCommunicationChannel",
    }
)
class As2CommunicationOptions(BaseModel):
    """As2CommunicationOptions

    :param as2_default_partner_settings: as2_default_partner_settings, defaults to None
    :type as2_default_partner_settings: As2SendSettings, optional
    :param as2_receive_options: as2_receive_options, defaults to None
    :type as2_receive_options: As2ReceiveOptions, optional
    :param as2_send_options: as2_send_options, defaults to None
    :type as2_send_options: As2SendOptions, optional
    :param as2_send_settings: as2_send_settings, defaults to None
    :type as2_send_settings: As2SendSettings, optional
    :param communication_setting: communication_setting, defaults to None
    :type communication_setting: As2CommunicationOptionsCommunicationSetting, optional
    :param shared_communication_channel: shared_communication_channel, defaults to None
    :type shared_communication_channel: SharedCommunicationChannel, optional
    """

    def __init__(
        self,
        as2_default_partner_settings: As2SendSettings = SENTINEL,
        as2_receive_options: As2ReceiveOptions = SENTINEL,
        as2_send_options: As2SendOptions = SENTINEL,
        as2_send_settings: As2SendSettings = SENTINEL,
        communication_setting: As2CommunicationOptionsCommunicationSetting = SENTINEL,
        shared_communication_channel: SharedCommunicationChannel = SENTINEL,
        **kwargs,
    ):
        """As2CommunicationOptions

        :param as2_default_partner_settings: as2_default_partner_settings, defaults to None
        :type as2_default_partner_settings: As2SendSettings, optional
        :param as2_receive_options: as2_receive_options, defaults to None
        :type as2_receive_options: As2ReceiveOptions, optional
        :param as2_send_options: as2_send_options, defaults to None
        :type as2_send_options: As2SendOptions, optional
        :param as2_send_settings: as2_send_settings, defaults to None
        :type as2_send_settings: As2SendSettings, optional
        :param communication_setting: communication_setting, defaults to None
        :type communication_setting: As2CommunicationOptionsCommunicationSetting, optional
        :param shared_communication_channel: shared_communication_channel, defaults to None
        :type shared_communication_channel: SharedCommunicationChannel, optional
        """
        if as2_default_partner_settings is not SENTINEL:
            self.as2_default_partner_settings = self._define_object(
                as2_default_partner_settings, As2SendSettings
            )
        if as2_receive_options is not SENTINEL:
            self.as2_receive_options = self._define_object(
                as2_receive_options, As2ReceiveOptions
            )
        if as2_send_options is not SENTINEL:
            self.as2_send_options = self._define_object(
                as2_send_options, As2SendOptions
            )
        if as2_send_settings is not SENTINEL:
            self.as2_send_settings = self._define_object(
                as2_send_settings, As2SendSettings
            )
        if communication_setting is not SENTINEL:
            self.communication_setting = self._enum_matching(
                communication_setting,
                As2CommunicationOptionsCommunicationSetting.list(),
                "communication_setting",
            )
        if shared_communication_channel is not SENTINEL:
            self.shared_communication_channel = self._define_object(
                shared_communication_channel, SharedCommunicationChannel
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "from_function": "fromFunction",
        "from_key": "fromKey",
        "to_function": "toFunction",
        "to_key": "toKey",
    }
)
class MapExtensionsFunctionMapping(BaseModel):
    """MapExtensionsFunctionMapping

    :param from_function: from_function, defaults to None
    :type from_function: str, optional
    :param from_key: from_key, defaults to None
    :type from_key: int, optional
    :param to_function: to_function, defaults to None
    :type to_function: str, optional
    :param to_key: to_key, defaults to None
    :type to_key: int, optional
    """

    def __init__(
        self,
        from_function: str = SENTINEL,
        from_key: int = SENTINEL,
        to_function: str = SENTINEL,
        to_key: int = SENTINEL,
        **kwargs
    ):
        """MapExtensionsFunctionMapping

        :param from_function: from_function, defaults to None
        :type from_function: str, optional
        :param from_key: from_key, defaults to None
        :type from_key: int, optional
        :param to_function: to_function, defaults to None
        :type to_function: str, optional
        :param to_key: to_key, defaults to None
        :type to_key: int, optional
        """
        if from_function is not SENTINEL:
            self.from_function = from_function
        if from_key is not SENTINEL:
            self.from_key = from_key
        if to_function is not SENTINEL:
            self.to_function = to_function
        if to_key is not SENTINEL:
            self.to_key = to_key
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .process_environment_attachment_expression import (
    ProcessEnvironmentAttachmentExpression,
    ProcessEnvironmentAttachmentExpressionGuard,
)
from .process_environment_attachment_simple_expression import (
    ProcessEnvironmentAttachmentSimpleExpression,
)
from .process_environment_attachment_grouping_expression import (
    ProcessEnvironmentAttachmentGroupingExpression,
)


@JsonMap({})
class ProcessEnvironmentAttachmentQueryConfigQueryFilter(BaseModel):
    """ProcessEnvironmentAttachmentQueryConfigQueryFilter

    :param expression: expression
    :type expression: ProcessEnvironmentAttachmentExpression
    """

    def __init__(self, expression: ProcessEnvironmentAttachmentExpression, **kwargs):
        """ProcessEnvironmentAttachmentQueryConfigQueryFilter

        :param expression: expression
        :type expression: ProcessEnvironmentAttachmentExpression
        """
        self.expression = ProcessEnvironmentAttachmentExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ProcessEnvironmentAttachmentQueryConfig(BaseModel):
    """ProcessEnvironmentAttachmentQueryConfig

    :param query_filter: query_filter
    :type query_filter: ProcessEnvironmentAttachmentQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ProcessEnvironmentAttachmentQueryConfigQueryFilter, **kwargs
    ):
        """ProcessEnvironmentAttachmentQueryConfig

        :param query_filter: query_filter
        :type query_filter: ProcessEnvironmentAttachmentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ProcessEnvironmentAttachmentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .pgp_certificates import PgpCertificates
from .connections import Connections
from .cross_references import CrossReferences
from .operations import Operations
from .override_process_properties import OverrideProcessProperties
from .properties import Properties
from .shared_communications import SharedCommunications
from .trading_partners import TradingPartners


@JsonMap(
    {
        "pgp_certificates": "PGPCertificates",
        "cross_references": "crossReferences",
        "environment_id": "environmentId",
        "extension_group_id": "extensionGroupId",
        "id_": "id",
        "process_properties": "processProperties",
        "shared_communications": "sharedCommunications",
        "trading_partners": "tradingPartners",
    }
)
class EnvironmentExtensions(BaseModel):
    """EnvironmentExtensions

    :param pgp_certificates: pgp_certificates, defaults to None
    :type pgp_certificates: PgpCertificates, optional
    :param connections: connections, defaults to None
    :type connections: Connections, optional
    :param cross_references: cross_references, defaults to None
    :type cross_references: CrossReferences, optional
    :param environment_id: The ID of the environment., defaults to None
    :type environment_id: str, optional
    :param extension_group_id: The synthesized ID of the process belonging to a multi-install integration pack to which the extension values apply, if applicable. For more information, see the section in the Working with Environment Extensions subtopic about multi-install integration packs., defaults to None
    :type extension_group_id: str, optional
    :param id_: The ID of the object. This can be either of the following:\<br /\> 1. The value of environmentId.\<br /\> 2. A conceptual ID synthesized from the environment ID \(environmentId\) and the ID of the multi-install integration pack to which the extension values apply \(extensionGroupId\)., defaults to None
    :type id_: str, optional
    :param operations: operations, defaults to None
    :type operations: Operations, optional
    :param partial: Supplied only in an UPDATE operation. \<br /\>-   If set to true, indicates that the request includes only a subset of environment extension values to update. \<br /\> -   If set to false, indicates that the request includes the full set of environment extension values to update. Values not included in the request are reset to use their default values., defaults to None
    :type partial: bool, optional
    :param process_properties: process_properties, defaults to None
    :type process_properties: OverrideProcessProperties, optional
    :param properties: properties, defaults to None
    :type properties: Properties, optional
    :param shared_communications: shared_communications, defaults to None
    :type shared_communications: SharedCommunications, optional
    :param trading_partners: trading_partners, defaults to None
    :type trading_partners: TradingPartners, optional
    """

    def __init__(
        self,
        pgp_certificates: PgpCertificates = SENTINEL,
        connections: Connections = SENTINEL,
        cross_references: CrossReferences = SENTINEL,
        operations: Operations = SENTINEL,
        process_properties: OverrideProcessProperties = SENTINEL,
        properties: Properties = SENTINEL,
        shared_communications: SharedCommunications = SENTINEL,
        trading_partners: TradingPartners = SENTINEL,
        environment_id: str = SENTINEL,
        extension_group_id: str = SENTINEL,
        id_: str = SENTINEL,
        partial: bool = SENTINEL,
        **kwargs,
    ):
        """EnvironmentExtensions

        :param pgp_certificates: pgp_certificates, defaults to None
        :type pgp_certificates: PgpCertificates, optional
        :param connections: connections, defaults to None
        :type connections: Connections, optional
        :param cross_references: cross_references, defaults to None
        :type cross_references: CrossReferences, optional
        :param environment_id: The ID of the environment., defaults to None
        :type environment_id: str, optional
        :param extension_group_id: The synthesized ID of the process belonging to a multi-install integration pack to which the extension values apply, if applicable. For more information, see the section in the Working with Environment Extensions subtopic about multi-install integration packs., defaults to None
        :type extension_group_id: str, optional
        :param id_: The ID of the object. This can be either of the following:\<br /\> 1. The value of environmentId.\<br /\> 2. A conceptual ID synthesized from the environment ID \(environmentId\) and the ID of the multi-install integration pack to which the extension values apply \(extensionGroupId\)., defaults to None
        :type id_: str, optional
        :param operations: operations, defaults to None
        :type operations: Operations, optional
        :param partial: Supplied only in an UPDATE operation. \<br /\>-   If set to true, indicates that the request includes only a subset of environment extension values to update. \<br /\> -   If set to false, indicates that the request includes the full set of environment extension values to update. Values not included in the request are reset to use their default values., defaults to None
        :type partial: bool, optional
        :param process_properties: process_properties, defaults to None
        :type process_properties: OverrideProcessProperties, optional
        :param properties: properties, defaults to None
        :type properties: Properties, optional
        :param shared_communications: shared_communications, defaults to None
        :type shared_communications: SharedCommunications, optional
        :param trading_partners: trading_partners, defaults to None
        :type trading_partners: TradingPartners, optional
        """
        if pgp_certificates is not SENTINEL:
            self.pgp_certificates = self._define_object(pgp_certificates, PgpCertificates)
        if connections is not SENTINEL:
            self.connections = self._define_object(connections, Connections)
        if cross_references is not SENTINEL:
            self.cross_references = self._define_object(cross_references, CrossReferences)
        if environment_id is not SENTINEL:
            self.environment_id = environment_id
        if extension_group_id is not SENTINEL:
            self.extension_group_id = extension_group_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if operations is not SENTINEL:
            self.operations = self._define_object(operations, Operations)
        if partial is not SENTINEL:
            self.partial = partial
        if process_properties is not SENTINEL:
            self.process_properties = self._define_object(
                process_properties, OverrideProcessProperties
            )
        if properties is not SENTINEL:
            self.properties = self._define_object(properties, Properties)
        if shared_communications is not SENTINEL:
            self.shared_communications = self._define_object(
                shared_communications, SharedCommunications
            )
        if trading_partners is not SENTINEL:
            self.trading_partners = self._define_object(trading_partners, TradingPartners)
        self._kwargs = kwargs




"""
An enum class containing all the possible environments for the SDK
"""
from enum import Enum
from urllib.parse import urlparse


class Environment(Enum):
    """The environments available for the SDK"""

    DEFAULT = "https://api.boomi.com/api/rest/v1/{accountId}"

    def __new__(cls, url):
        parsed_url = urlparse(url)
        if not all([parsed_url.scheme, parsed_url.netloc]):
            raise ValueError(
                f"Environment url [{url}] is not valid. Please use the following format https://api.example.com"
            )

        obj = object.__new__(cls)
        obj._value_ = url
        obj._url = url
        return obj

    @property
    def url(self):
        """Get the base URL for this environment"""
        return self._url




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .map_extensions_configuration import MapExtensionsConfiguration
from .map_extensions_inputs import MapExtensionsInputs
from .map_extensions_outputs import MapExtensionsOutputs


class MapExtensionsFunctionStepCacheType(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "None"
    :vartype NONE: str
    :cvar BYDOCUMENT: "ByDocument"
    :vartype BYDOCUMENT: str
    :cvar BYMAP: "ByMap"
    :vartype BYMAP: str
    """

    NONE = "None"
    BYDOCUMENT = "ByDocument"
    BYMAP = "ByMap"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                MapExtensionsFunctionStepCacheType._member_map_.values(),
            )
        )


class MapExtensionsFunctionStepType(Enum):
    """An enumeration representing different categories.

    :cvar COUNT: "Count"
    :vartype COUNT: str
    :cvar CURRENTDATE: "CurrentDate"
    :vartype CURRENTDATE: str
    :cvar DATEFORMAT: "DateFormat"
    :vartype DATEFORMAT: str
    :cvar LEFTTRIM: "LeftTrim"
    :vartype LEFTTRIM: str
    :cvar LINEITEMINCREMENT: "LineItemIncrement"
    :vartype LINEITEMINCREMENT: str
    :cvar MATHABS: "MathABS"
    :vartype MATHABS: str
    :cvar MATHADD: "MathAdd"
    :vartype MATHADD: str
    :cvar MATHCEIL: "MathCeil"
    :vartype MATHCEIL: str
    :cvar MATHDIVIDE: "MathDivide"
    :vartype MATHDIVIDE: str
    :cvar MATHFLOOR: "MathFloor"
    :vartype MATHFLOOR: str
    :cvar MATHMULTIPLY: "MathMultiply"
    :vartype MATHMULTIPLY: str
    :cvar MATHSETPRECISION: "MathSetPrecision"
    :vartype MATHSETPRECISION: str
    :cvar MATHSUBTRACT: "MathSubtract"
    :vartype MATHSUBTRACT: str
    :cvar NUMBERFORMAT: "NumberFormat"
    :vartype NUMBERFORMAT: str
    :cvar PROPERTYGET: "PropertyGet"
    :vartype PROPERTYGET: str
    :cvar PROPERTYSET: "PropertySet"
    :vartype PROPERTYSET: str
    :cvar RIGHTTRIM: "RightTrim"
    :vartype RIGHTTRIM: str
    :cvar RUNNINGTOTAL: "RunningTotal"
    :vartype RUNNINGTOTAL: str
    :cvar STRINGAPPEND: "StringAppend"
    :vartype STRINGAPPEND: str
    :cvar STRINGPREPEND: "StringPrepend"
    :vartype STRINGPREPEND: str
    :cvar STRINGREMOVE: "StringRemove"
    :vartype STRINGREMOVE: str
    :cvar STRINGREPLACE: "StringReplace"
    :vartype STRINGREPLACE: str
    :cvar STRINGTOLOWER: "StringToLower"
    :vartype STRINGTOLOWER: str
    :cvar STRINGTOUPPER: "StringToUpper"
    :vartype STRINGTOUPPER: str
    :cvar SUM: "Sum"
    :vartype SUM: str
    :cvar TRIMWHITESPACE: "TrimWhitespace"
    :vartype TRIMWHITESPACE: str
    :cvar STRINGCONCAT: "StringConcat"
    :vartype STRINGCONCAT: str
    :cvar STRINGSPLIT: "StringSplit"
    :vartype STRINGSPLIT: str
    :cvar SEQUENTIALVALUE: "SequentialValue"
    :vartype SEQUENTIALVALUE: str
    :cvar SIMPLELOOKUP: "SimpleLookup"
    :vartype SIMPLELOOKUP: str
    :cvar DOCUMENTPROPERTYGET: "DocumentPropertyGet"
    :vartype DOCUMENTPROPERTYGET: str
    :cvar DOCUMENTPROPERTYSET: "DocumentPropertySet"
    :vartype DOCUMENTPROPERTYSET: str
    :cvar CROSSREFLOOKUP: "CrossRefLookup"
    :vartype CROSSREFLOOKUP: str
    :cvar DOCUMENTCACHELOOKUP: "DocumentCacheLookup"
    :vartype DOCUMENTCACHELOOKUP: str
    :cvar CUSTOMSCRIPTING: "CustomScripting"
    :vartype CUSTOMSCRIPTING: str
    :cvar USERDEFINED: "UserDefined"
    :vartype USERDEFINED: str
    :cvar JAPANESECHARACTERCONVERSION: "JapaneseCharacterConversion"
    :vartype JAPANESECHARACTERCONVERSION: str
    """

    COUNT = "Count"
    CURRENTDATE = "CurrentDate"
    DATEFORMAT = "DateFormat"
    LEFTTRIM = "LeftTrim"
    LINEITEMINCREMENT = "LineItemIncrement"
    MATHABS = "MathABS"
    MATHADD = "MathAdd"
    MATHCEIL = "MathCeil"
    MATHDIVIDE = "MathDivide"
    MATHFLOOR = "MathFloor"
    MATHMULTIPLY = "MathMultiply"
    MATHSETPRECISION = "MathSetPrecision"
    MATHSUBTRACT = "MathSubtract"
    NUMBERFORMAT = "NumberFormat"
    PROPERTYGET = "PropertyGet"
    PROPERTYSET = "PropertySet"
    RIGHTTRIM = "RightTrim"
    RUNNINGTOTAL = "RunningTotal"
    STRINGAPPEND = "StringAppend"
    STRINGPREPEND = "StringPrepend"
    STRINGREMOVE = "StringRemove"
    STRINGREPLACE = "StringReplace"
    STRINGTOLOWER = "StringToLower"
    STRINGTOUPPER = "StringToUpper"
    SUM = "Sum"
    TRIMWHITESPACE = "TrimWhitespace"
    STRINGCONCAT = "StringConcat"
    STRINGSPLIT = "StringSplit"
    SEQUENTIALVALUE = "SequentialValue"
    SIMPLELOOKUP = "SimpleLookup"
    DOCUMENTPROPERTYGET = "DocumentPropertyGet"
    DOCUMENTPROPERTYSET = "DocumentPropertySet"
    CROSSREFLOOKUP = "CrossRefLookup"
    DOCUMENTCACHELOOKUP = "DocumentCacheLookup"
    CUSTOMSCRIPTING = "CustomScripting"
    USERDEFINED = "UserDefined"
    JAPANESECHARACTERCONVERSION = "JapaneseCharacterConversion"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, MapExtensionsFunctionStepType._member_map_.values())
        )


@JsonMap(
    {
        "configuration": "Configuration",
        "inputs": "Inputs",
        "outputs": "Outputs",
        "cache_type": "cacheType",
        "id_": "id",
        "type_": "type",
    }
)
class MapExtensionsFunctionStep(BaseModel):
    """MapExtensionsFunctionStep

    :param configuration: configuration
    :type configuration: MapExtensionsConfiguration
    :param inputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
    :type inputs: MapExtensionsInputs
    :param outputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. See the following row for more information. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
    :type outputs: MapExtensionsOutputs
    :param cache_type: cache_type, defaults to None
    :type cache_type: MapExtensionsFunctionStepCacheType, optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param position: position, defaults to None
    :type position: int, optional
    :param type_: type_, defaults to None
    :type type_: MapExtensionsFunctionStepType, optional
    """

    def __init__(
        self,
        configuration: MapExtensionsConfiguration,
        inputs: MapExtensionsInputs,
        outputs: MapExtensionsOutputs,
        cache_type: MapExtensionsFunctionStepCacheType = SENTINEL,
        id_: str = SENTINEL,
        position: int = SENTINEL,
        type_: MapExtensionsFunctionStepType = SENTINEL,
        **kwargs,
    ):
        """MapExtensionsFunctionStep

        :param configuration: configuration
        :type configuration: MapExtensionsConfiguration
        :param inputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
        :type inputs: MapExtensionsInputs
        :param outputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. See the following row for more information. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
        :type outputs: MapExtensionsOutputs
        :param cache_type: cache_type, defaults to None
        :type cache_type: MapExtensionsFunctionStepCacheType, optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param position: position, defaults to None
        :type position: int, optional
        :param type_: type_, defaults to None
        :type type_: MapExtensionsFunctionStepType, optional
        """
        self.configuration = self._define_object(
            configuration, MapExtensionsConfiguration
        )
        self.inputs = self._define_object(inputs, MapExtensionsInputs)
        self.outputs = self._define_object(outputs, MapExtensionsOutputs)
        if cache_type is not SENTINEL:
            self.cache_type = self._enum_matching(
                cache_type, MapExtensionsFunctionStepCacheType.list(), "cache_type"
            )
        if id_ is not SENTINEL:
            self.id_ = id_
        if position is not SENTINEL:
            self.position = position
        if type_ is not SENTINEL:
            self.type_ = self._enum_matching(
                type_, MapExtensionsFunctionStepType.list(), "type_"
            )
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .private_certificate import PrivateCertificate


class Sslmode(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "none"
    :vartype NONE: str
    :cvar EXPLICIT: "explicit"
    :vartype EXPLICIT: str
    :cvar IMPLICIT: "implicit"
    :vartype IMPLICIT: str
    """

    NONE = "none"
    EXPLICIT = "explicit"
    IMPLICIT = "implicit"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Sslmode._member_map_.values()))


@JsonMap(
    {
        "client_ssl_certificate": "clientSSLCertificate",
        "use_client_authentication": "useClientAuthentication",
    }
)
class FtpsslOptions(BaseModel):
    """FtpsslOptions

    :param client_ssl_certificate: client_ssl_certificate
    :type client_ssl_certificate: PrivateCertificate
    :param sslmode: sslmode, defaults to None
    :type sslmode: Sslmode, optional
    :param use_client_authentication: use_client_authentication, defaults to None
    :type use_client_authentication: bool, optional
    """

    def __init__(
        self,
        client_ssl_certificate: PrivateCertificate,
        sslmode: Sslmode = SENTINEL,
        use_client_authentication: bool = SENTINEL,
        **kwargs,
    ):
        """FtpsslOptions

        :param client_ssl_certificate: client_ssl_certificate
        :type client_ssl_certificate: PrivateCertificate
        :param sslmode: sslmode, defaults to None
        :type sslmode: Sslmode, optional
        :param use_client_authentication: use_client_authentication, defaults to None
        :type use_client_authentication: bool, optional
        """
        self.client_ssl_certificate = self._define_object(
            client_ssl_certificate, PrivateCertificate
        )
        if sslmode is not SENTINEL:
            self.sslmode = self._enum_matching(sslmode, Sslmode.list(), "sslmode")
        if use_client_authentication is not SENTINEL:
            self.use_client_authentication = use_client_authentication
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .resource import Resource


@JsonMap({"resource": "Resource"})
class Resources(BaseModel):
    """Resources

    :param resource: resource, defaults to None
    :type resource: List[Resource], optional
    """

    def __init__(self, resource: List[Resource] = SENTINEL, **kwargs):
        """Resources

        :param resource: resource, defaults to None
        :type resource: List[Resource], optional
        """
        if resource is not SENTINEL:
            self.resource = self._define_list(resource, Resource)
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class DocumentStatus(Enum):
    """An enumeration representing different categories.

    :cvar ANY: "ANY"
    :vartype ANY: str
    :cvar SUCCESS: "SUCCESS"
    :vartype SUCCESS: str
    :cvar ERROR: "ERROR"
    :vartype ERROR: str
    """

    ANY = "ANY"
    SUCCESS = "SUCCESS"
    ERROR = "ERROR"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, DocumentStatus._member_map_.values()))


@JsonMap({"document_status": "documentStatus"})
class AllDocuments(BaseModel):
    """You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both.

     Indicates that the Rerun Document operation reruns all documents in the original run.

    When using AllDocuments in a request, you must also specify a `documentStatus` value.

    :param document_status: - A value of ANY reruns all documents in the specified process run (in other words, the originalExecutionID).  - A value of SUCCESS returns successfully run documents in the process run.  - A value of ERROR returns documents that unsuccessfully ran in the process run., defaults to None
    :type document_status: DocumentStatus, optional
    """

    def __init__(self, document_status: DocumentStatus = SENTINEL, **kwargs):
        """You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both.

         Indicates that the Rerun Document operation reruns all documents in the original run.

        When using AllDocuments in a request, you must also specify a `documentStatus` value.

        :param document_status: - A value of ANY reruns all documents in the specified process run (in other words, the originalExecutionID).  - A value of SUCCESS returns successfully run documents in the process run.  - A value of ERROR returns documents that unsuccessfully ran in the process run., defaults to None
        :type document_status: DocumentStatus, optional
        """
        if document_status is not SENTINEL:
            self.document_status = self._enum_matching(
                document_status, DocumentStatus.list(), "document_status"
            )
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class As2Workload(Enum):
    """An enumeration representing different categories.

    :cvar GENERAL: "GENERAL"
    :vartype GENERAL: str
    :cvar LOWLATENCYDEBUG: "LOW_LATENCY_DEBUG"
    :vartype LOWLATENCYDEBUG: str
    """

    GENERAL = "GENERAL"
    LOWLATENCYDEBUG = "LOW_LATENCY_DEBUG"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, As2Workload._member_map_.values()))


class FlowControlParallelProcessTypeOverride(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "NONE"
    :vartype NONE: str
    :cvar THREADS: "THREADS"
    :vartype THREADS: str
    :cvar PROCESSES: "PROCESSES"
    :vartype PROCESSES: str
    """

    NONE = "NONE"
    THREADS = "THREADS"
    PROCESSES = "PROCESSES"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                FlowControlParallelProcessTypeOverride._member_map_.values(),
            )
        )


class HttpWorkload(Enum):
    """An enumeration representing different categories.

    :cvar GENERAL: "GENERAL"
    :vartype GENERAL: str
    :cvar LOWLATENCYDEBUG: "LOW_LATENCY_DEBUG"
    :vartype LOWLATENCYDEBUG: str
    :cvar LOWLATENCY: "LOW_LATENCY"
    :vartype LOWLATENCY: str
    """

    GENERAL = "GENERAL"
    LOWLATENCYDEBUG = "LOW_LATENCY_DEBUG"
    LOWLATENCY = "LOW_LATENCY"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, HttpWorkload._member_map_.values()))


@JsonMap(
    {
        "account_disk_usage": "accountDiskUsage",
        "as2_workload": "as2Workload",
        "atom_input_size": "atomInputSize",
        "atom_output_overflow_size": "atomOutputOverflowSize",
        "atom_working_overflow_size": "atomWorkingOverflowSize",
        "cloud_account_execution_limit": "cloudAccountExecutionLimit",
        "cloud_account_execution_warning_offset": "cloudAccountExecutionWarningOffset",
        "container_id": "containerId",
        "download_runnerlogs": "downloadRunnerlogs",
        "enable_account_data_archiving": "enableAccountDataArchiving",
        "enable_atom_worker_warmup": "enableAtomWorkerWarmup",
        "flow_control_parallel_process_type_override": "flowControlParallelProcessTypeOverride",
        "http_request_rate": "httpRequestRate",
        "http_workload": "httpWorkload",
        "listener_max_concurrent_executions": "listenerMaxConcurrentExecutions",
        "max_connector_track_docs": "maxConnectorTrackDocs",
        "min_numberof_atom_workers": "minNumberofAtomWorkers",
        "numberof_atom_workers": "numberofAtomWorkers",
        "queue_incoming_message_rate_limit": "queueIncomingMessageRateLimit",
        "session_id": "sessionId",
        "status_code": "statusCode",
        "test_mode_max_doc_bytes": "testModeMaxDocBytes",
        "test_mode_max_docs": "testModeMaxDocs",
        "worker_elastic_scaling_threshold": "workerElasticScalingThreshold",
        "worker_max_execution_time": "workerMaxExecutionTime",
        "worker_max_general_execution_time": "workerMaxGeneralExecutionTime",
        "worker_max_queued_executions": "workerMaxQueuedExecutions",
        "worker_max_running_executions": "workerMaxRunningExecutions",
        "worker_queued_execution_timeout": "workerQueuedExecutionTimeout",
    }
)
class AccountCloudAttachmentProperties(BaseModel):
    """AccountCloudAttachmentProperties

    :param account_disk_usage: Represented in bytes, this property sets the size limit for an account that uses the private Runtime cloud., defaults to None
    :type account_disk_usage: int, optional
    :param as2_workload: Used to select the process run mode for AS2 listener processes.Accepted values:\<br /\>1. inherited — \(Default\) The setting is inherited from the Runtime cluster.\<br /\>2. general — The default process mode for all new processes.\<br /\>3. low\_latency\_debug — All AS2 listener processes use an execution worker.  \>**Note:** After you change this property value you must restart the Runtime cloud cluster or Runtime. \<br /\>\<br /\>If you select Low\_Latency\_Debug, Trading Partner components that use AS2 listeners also use that run mode., defaults to None
    :type as2_workload: As2Workload, optional
    :param atom_input_size: Represented in bytes. For Web Services Server listener processes, limits the input size of a web service request. If reaching this limit, it rejects the request.For Flow Services Server listener processes, limits the request size and response size. If reaching this limit, it rejects the request. If reaching this limit, it rejects the request with a 400 error code. If reaching the limit on a response, it rejects the request with a 503 error code., defaults to None
    :type atom_input_size: int, optional
    :param atom_output_overflow_size: Represented in bytes, if specified, this value must be a positive number. For Runtime worker processes, this property limits the number of bytes per output datastore maintained in memory before overflowing to disk., defaults to None
    :type atom_output_overflow_size: int, optional
    :param atom_working_overflow_size: Represented in bytes. For Runtime worker processes, this property limits the number of bytes per working datastore maintained in memory before overflowing to disk., defaults to None
    :type atom_working_overflow_size: int, optional
    :param cloud_account_execution_limit: The total number of concurrent runs allowed. If specified, this value must be a positive number. If this field does not contain a value, there is no limit. On a Cloud, for this limit to take effect, you must also set the **Cloud Partition Size** property. The **Cloud Partition Size** property is set in the **Properties** panel, under **Runtime Management**., defaults to None
    :type cloud_account_execution_limit: int, optional
    :param cloud_account_execution_warning_offset: If specified, this value must be a positive number. If this field does not contain a value, it does not generate a warning. This value is subtracted from the Account Concurrent Execution Limit to determine when the Runtime generates a warning that the account is close to exceeding its number of concurrent runs. For example, if this property is set to 5 and the Account Concurrent Execution Limit is set to 20, the Runtime generates a warning in the container log when the account exceeds 15 concurrent runs., defaults to None
    :type cloud_account_execution_warning_offset: int, optional
    :param container_id: container_id, defaults to None
    :type container_id: str, optional
    :param download_runnerlogs: Runtime cloud owners can set this property to give account users or specific tenants the ability to download Runtime Worker log files from **Runtime Management** and run artifacts from **Process Reporting**.Accepted values:\<br /\>1. Inherited \(false\) — The setting is inherited from the Runtime cloud cluster, and the feature is turned off.\<br /\>2. False — The feature is not enabled, and users cannot download Runtime Worker logs or run artifacts.\<br /\>3. True — \(default\) The feature is enabled. Users can download Runtime Worker logs and run artifacts.  \>**Note:** This property is turned on automatically for public Runtime clouds., defaults to None
    :type download_runnerlogs: bool, optional
    :param enable_account_data_archiving: Accepted values:\<br /\>1. inherited- \(plus the value of the setting being inherited. For example, inherited\(true\) or inherited\(false\)\) indicates that the property inherits what is set in the Runtime cloud cluster.\<br /\>2. true- If true or Inherited \(true\), the owner of the selected attached Runtime can enable [processed document archiving](https://help.boomi.com/bundle/integration/page/c-atm-Processed_document_archiving.html).\<br /\>3. false- indicates that the feature is not enabled., defaults to None
    :type enable_account_data_archiving: bool, optional
    :param enable_atom_worker_warmup: Accepted values:\<br /\>1. inherited- indicating that the property inherits what is set in the Runtime cloud cluster.\<br /\>2. true- indicating that when an execution worker is within 30 minutes of shutting down, the Cloud starts \(“warm-up”\) another Runtime worker to replace it.\<br /\>3. false- indicates that you did not enable the feature. You can use this feature if you provision your account with at least one Runtime worker. If you provision your account with more than one execution worker, and if there are multiple execution workers within 30 minutes of shutting down, then by default it replaces only one execution worker. However, if one of those execution workers has a load greater than “LOW”, then it is replaced even though there is another execution worker running. If you have set the **Minimum Execution Workers** property, then it replaces the appropriate number of execution workers so that you do not go below the minimum., defaults to None
    :type enable_atom_worker_warmup: bool, optional
    :param flow_control_parallel_process_type_override: Overrides the **Parallel Process Type** setting in **Flow Control** shapes at a global Runtime cloud or Attachment Quota level. You can set the property only if you are a private Runtime cloud owner.The default value is NONE., defaults to None
    :type flow_control_parallel_process_type_override: FlowControlParallelProcessTypeOverride, optional
    :param http_request_rate: Limits the number of Web Services Server requests per second. This limitation is per node in the Runtime cloud, and per Cloud or Runtime attachment \(*not* per account\).If it exceeds this value, callers receive a 503 error. After you change this property value you must restart the Runtime cloud cluster or Runtime.   \>**Note:** If you set an HTTP Request Rate value, API Management uses this limit before the values specified in the API Contract for Rate Limit or Quota Limit., defaults to None
    :type http_request_rate: int, optional
    :param http_workload: Accepted values:\<br /\>1. inherited- \(plus the value of the setting being inherited. For example, inherited\(true\) or inherited\(false\)\) indicating that the property inherits what is set in the Runtime cloud cluster.\<br /\>2. general- The default process mode for all new processes. General mode processes do not use an execution worker, but Low\_Latency mode processes do use one.\<br /\>3. low\_latency- All Web Services Server processes use an execution worker and run in Low\_Latency mode.\<br /\>4. low\_latency\_debug- All Web Services Server processes use an execution worker. Processes configured in Generalmode run in Low\_Latency\_Debug mode., defaults to None
    :type http_workload: HttpWorkload, optional
    :param listener_max_concurrent_executions: The maximum number of concurrent runs allowed per listener — that is, the maximum listener pool size., defaults to None
    :type listener_max_concurrent_executions: int, optional
    :param max_connector_track_docs: You must set a positive integer less than or equal to 10,000. For example, in a process run, the maximum number of connector tracking generated documents for a single connector step.After reaching this limit, it does not report more tracking documents to the platform for that step in that run., defaults to None
    :type max_connector_track_docs: int, optional
    :param min_numberof_atom_workers: The minimum number of execution workers that run continuously in the Cloud.If this property is set to a number greater than zero, then a few minutes after the Cloud starts and stabilizes, this number of execution workers begin to run. Their starting is not dependent upon receiving incoming process run requests. This behavior is like having multiple runners at the starting line of a race. All runners are ready and start to run as soon as the starter pistol is fired. This property works in conjunction with the **Execution Worker Warmup Enabled** property. If you set **Minimum Execution Workers** \\> 0, then it enables Execution Worker Warmup behavior. As your minimum number of execution workers reach the end of their life span, they are replaced with new execution workers. If the load on active execution workers drop, the Runtime cloud reduces the number of execution workers to the value you set for this property.   \>**Note:** Setting this property to a number greater than the number of provisioned execution workers in your account does not cause additional execution workers to run. If you would like to have additional execution workers provisioned in your account, contact the Support team., defaults to None
    :type min_numberof_atom_workers: int, optional
    :param numberof_atom_workers: Allocated number of execution workers., defaults to None
    :type numberof_atom_workers: int, optional
    :param queue_incoming_message_rate_limit: The maximum number of requests the attachment is allowed to send to the Shared Queue Server per minute. The limit is only enforced if the Incoming Message Rate Limit is set in the underlying cloud Queue Shared Server. If a value is not set or is less than 1, the Shared Queue Server limit is used. The message is rejected if the limit is exceeded and should be retried from within the integration process. The limit is enforced per cloud node., defaults to None
    :type queue_incoming_message_rate_limit: int, optional
    :param session_id: session_id, defaults to None
    :type session_id: str, optional
    :param status_code: status_code, defaults to None
    :type status_code: int, optional
    :param test_mode_max_doc_bytes: Represented in bytes, the maximum aggregate data size across the run of a process in test mode. A negative value means there is no maximum.This field is present only if you enable the enhanced test mode feature in the account. If you want to enable this feature, contact your sales representative., defaults to None
    :type test_mode_max_doc_bytes: int, optional
    :param test_mode_max_docs: The maximum number of files \(documents\) per inbound connector shape during the run of a process in test mode. A negative value means there is no maximum.This field is present only if you enable enhanced test mode feature in the account., defaults to None
    :type test_mode_max_docs: int, optional
    :param worker_elastic_scaling_threshold: worker_elastic_scaling_threshold, defaults to None
    :type worker_elastic_scaling_threshold: int, optional
    :param worker_max_execution_time: Maximum run time in milliseconds for Execution worker processes. For example, an accepted value is 30000. After this amount of time passes, a 522 HTTP status code message is returned to the client saying that the process exceeds the time limit and then cancels the process run. After you change this property value, you must restart the Runtime cloud cluster or Runtime., defaults to None
    :type worker_max_execution_time: int, optional
    :param worker_max_general_execution_time: Maximum run time in milliseconds for Execution worker processes that use Low\_Latency\_Debug mode. For example, an accepted value is 60000. After you change this property value, you must restart the Runtime cloud cluster or Runtime., defaults to None
    :type worker_max_general_execution_time: int, optional
    :param worker_max_queued_executions: Maximum number of extra processes that can queue in a single Execution worker, when the maximum number of processes is running. If specified, this value must be a positive number. After you change this property value you must restart the Runtime cloud cluster or Runtime., defaults to None
    :type worker_max_queued_executions: int, optional
    :param worker_max_running_executions: The maximum number of simultaneous processes that can run in a single Execution worker.After you change this property value, you must restart the Runtime cloud cluster or Runtime., defaults to None
    :type worker_max_running_executions: int, optional
    :param worker_queued_execution_timeout: Maximum time that a job in the queue can wait to run. After this amount of time, the job fails with a time-out exception.For example, an accepted value is 0:10:00. After you change this property value you must restart the Runtime cloud cluster or Runtime., defaults to None
    :type worker_queued_execution_timeout: int, optional
    """

    def __init__(
        self,
        account_disk_usage: int = SENTINEL,
        as2_workload: As2Workload = SENTINEL,
        atom_input_size: int = SENTINEL,
        atom_output_overflow_size: int = SENTINEL,
        atom_working_overflow_size: int = SENTINEL,
        cloud_account_execution_limit: int = SENTINEL,
        cloud_account_execution_warning_offset: int = SENTINEL,
        container_id: str = SENTINEL,
        download_runnerlogs: bool = SENTINEL,
        enable_account_data_archiving: bool = SENTINEL,
        enable_atom_worker_warmup: bool = SENTINEL,
        flow_control_parallel_process_type_override: FlowControlParallelProcessTypeOverride = SENTINEL,
        http_request_rate: int = SENTINEL,
        http_workload: HttpWorkload = SENTINEL,
        listener_max_concurrent_executions: int = SENTINEL,
        max_connector_track_docs: int = SENTINEL,
        min_numberof_atom_workers: int = SENTINEL,
        numberof_atom_workers: int = SENTINEL,
        queue_incoming_message_rate_limit: int = SENTINEL,
        session_id: str = SENTINEL,
        status_code: int = SENTINEL,
        test_mode_max_doc_bytes: int = SENTINEL,
        test_mode_max_docs: int = SENTINEL,
        worker_elastic_scaling_threshold: int = SENTINEL,
        worker_max_execution_time: int = SENTINEL,
        worker_max_general_execution_time: int = SENTINEL,
        worker_max_queued_executions: int = SENTINEL,
        worker_max_running_executions: int = SENTINEL,
        worker_queued_execution_timeout: int = SENTINEL,
        **kwargs
    ):
        """AccountCloudAttachmentProperties

        :param account_disk_usage: Represented in bytes, this property sets the size limit for an account that uses the private Runtime cloud., defaults to None
        :type account_disk_usage: int, optional
        :param as2_workload: Used to select the process run mode for AS2 listener processes.Accepted values:\<br /\>1. inherited — \(Default\) The setting is inherited from the Runtime cluster.\<br /\>2. general — The default process mode for all new processes.\<br /\>3. low\_latency\_debug — All AS2 listener processes use an execution worker.  \>**Note:** After you change this property value you must restart the Runtime cloud cluster or Runtime. \<br /\>\<br /\>If you select Low\_Latency\_Debug, Trading Partner components that use AS2 listeners also use that run mode., defaults to None
        :type as2_workload: As2Workload, optional
        :param atom_input_size: Represented in bytes. For Web Services Server listener processes, limits the input size of a web service request. If reaching this limit, it rejects the request.For Flow Services Server listener processes, limits the request size and response size. If reaching this limit, it rejects the request. If reaching this limit, it rejects the request with a 400 error code. If reaching the limit on a response, it rejects the request with a 503 error code., defaults to None
        :type atom_input_size: int, optional
        :param atom_output_overflow_size: Represented in bytes, if specified, this value must be a positive number. For Runtime worker processes, this property limits the number of bytes per output datastore maintained in memory before overflowing to disk., defaults to None
        :type atom_output_overflow_size: int, optional
        :param atom_working_overflow_size: Represented in bytes. For Runtime worker processes, this property limits the number of bytes per working datastore maintained in memory before overflowing to disk., defaults to None
        :type atom_working_overflow_size: int, optional
        :param cloud_account_execution_limit: The total number of concurrent runs allowed. If specified, this value must be a positive number. If this field does not contain a value, there is no limit. On a Cloud, for this limit to take effect, you must also set the **Cloud Partition Size** property. The **Cloud Partition Size** property is set in the **Properties** panel, under **Runtime Management**., defaults to None
        :type cloud_account_execution_limit: int, optional
        :param cloud_account_execution_warning_offset: If specified, this value must be a positive number. If this field does not contain a value, it does not generate a warning. This value is subtracted from the Account Concurrent Execution Limit to determine when the Runtime generates a warning that the account is close to exceeding its number of concurrent runs. For example, if this property is set to 5 and the Account Concurrent Execution Limit is set to 20, the Runtime generates a warning in the container log when the account exceeds 15 concurrent runs., defaults to None
        :type cloud_account_execution_warning_offset: int, optional
        :param container_id: container_id, defaults to None
        :type container_id: str, optional
        :param download_runnerlogs: Runtime cloud owners can set this property to give account users or specific tenants the ability to download Runtime Worker log files from **Runtime Management** and run artifacts from **Process Reporting**.Accepted values:\<br /\>1. Inherited \(false\) — The setting is inherited from the Runtime cloud cluster, and the feature is turned off.\<br /\>2. False — The feature is not enabled, and users cannot download Runtime Worker logs or run artifacts.\<br /\>3. True — \(default\) The feature is enabled. Users can download Runtime Worker logs and run artifacts.  \>**Note:** This property is turned on automatically for public Runtime clouds., defaults to None
        :type download_runnerlogs: bool, optional
        :param enable_account_data_archiving: Accepted values:\<br /\>1. inherited- \(plus the value of the setting being inherited. For example, inherited\(true\) or inherited\(false\)\) indicates that the property inherits what is set in the Runtime cloud cluster.\<br /\>2. true- If true or Inherited \(true\), the owner of the selected attached Runtime can enable [processed document archiving](https://help.boomi.com/bundle/integration/page/c-atm-Processed_document_archiving.html).\<br /\>3. false- indicates that the feature is not enabled., defaults to None
        :type enable_account_data_archiving: bool, optional
        :param enable_atom_worker_warmup: Accepted values:\<br /\>1. inherited- indicating that the property inherits what is set in the Runtime cloud cluster.\<br /\>2. true- indicating that when an execution worker is within 30 minutes of shutting down, the Cloud starts \(“warm-up”\) another Runtime worker to replace it.\<br /\>3. false- indicates that you did not enable the feature. You can use this feature if you provision your account with at least one Runtime worker. If you provision your account with more than one execution worker, and if there are multiple execution workers within 30 minutes of shutting down, then by default it replaces only one execution worker. However, if one of those execution workers has a load greater than “LOW”, then it is replaced even though there is another execution worker running. If you have set the **Minimum Execution Workers** property, then it replaces the appropriate number of execution workers so that you do not go below the minimum., defaults to None
        :type enable_atom_worker_warmup: bool, optional
        :param flow_control_parallel_process_type_override: Overrides the **Parallel Process Type** setting in **Flow Control** shapes at a global Runtime cloud or Attachment Quota level. You can set the property only if you are a private Runtime cloud owner.The default value is NONE., defaults to None
        :type flow_control_parallel_process_type_override: FlowControlParallelProcessTypeOverride, optional
        :param http_request_rate: Limits the number of Web Services Server requests per second. This limitation is per node in the Runtime cloud, and per Cloud or Runtime attachment \(*not* per account\).If it exceeds this value, callers receive a 503 error. After you change this property value you must restart the Runtime cloud cluster or Runtime.   \>**Note:** If you set an HTTP Request Rate value, API Management uses this limit before the values specified in the API Contract for Rate Limit or Quota Limit., defaults to None
        :type http_request_rate: int, optional
        :param http_workload: Accepted values:\<br /\>1. inherited- \(plus the value of the setting being inherited. For example, inherited\(true\) or inherited\(false\)\) indicating that the property inherits what is set in the Runtime cloud cluster.\<br /\>2. general- The default process mode for all new processes. General mode processes do not use an execution worker, but Low\_Latency mode processes do use one.\<br /\>3. low\_latency- All Web Services Server processes use an execution worker and run in Low\_Latency mode.\<br /\>4. low\_latency\_debug- All Web Services Server processes use an execution worker. Processes configured in Generalmode run in Low\_Latency\_Debug mode., defaults to None
        :type http_workload: HttpWorkload, optional
        :param listener_max_concurrent_executions: The maximum number of concurrent runs allowed per listener — that is, the maximum listener pool size., defaults to None
        :type listener_max_concurrent_executions: int, optional
        :param max_connector_track_docs: You must set a positive integer less than or equal to 10,000. For example, in a process run, the maximum number of connector tracking generated documents for a single connector step.After reaching this limit, it does not report more tracking documents to the platform for that step in that run., defaults to None
        :type max_connector_track_docs: int, optional
        :param min_numberof_atom_workers: The minimum number of execution workers that run continuously in the Cloud.If this property is set to a number greater than zero, then a few minutes after the Cloud starts and stabilizes, this number of execution workers begin to run. Their starting is not dependent upon receiving incoming process run requests. This behavior is like having multiple runners at the starting line of a race. All runners are ready and start to run as soon as the starter pistol is fired. This property works in conjunction with the **Execution Worker Warmup Enabled** property. If you set **Minimum Execution Workers** \\> 0, then it enables Execution Worker Warmup behavior. As your minimum number of execution workers reach the end of their life span, they are replaced with new execution workers. If the load on active execution workers drop, the Runtime cloud reduces the number of execution workers to the value you set for this property.   \>**Note:** Setting this property to a number greater than the number of provisioned execution workers in your account does not cause additional execution workers to run. If you would like to have additional execution workers provisioned in your account, contact the Support team., defaults to None
        :type min_numberof_atom_workers: int, optional
        :param numberof_atom_workers: Allocated number of execution workers., defaults to None
        :type numberof_atom_workers: int, optional
        :param queue_incoming_message_rate_limit: The maximum number of requests the attachment is allowed to send to the Shared Queue Server per minute. The limit is only enforced if the Incoming Message Rate Limit is set in the underlying cloud Queue Shared Server. If a value is not set or is less than 1, the Shared Queue Server limit is used. The message is rejected if the limit is exceeded and should be retried from within the integration process. The limit is enforced per cloud node., defaults to None
        :type queue_incoming_message_rate_limit: int, optional
        :param session_id: session_id, defaults to None
        :type session_id: str, optional
        :param status_code: status_code, defaults to None
        :type status_code: int, optional
        :param test_mode_max_doc_bytes: Represented in bytes, the maximum aggregate data size across the run of a process in test mode. A negative value means there is no maximum.This field is present only if you enable the enhanced test mode feature in the account. If you want to enable this feature, contact your sales representative., defaults to None
        :type test_mode_max_doc_bytes: int, optional
        :param test_mode_max_docs: The maximum number of files \(documents\) per inbound connector shape during the run of a process in test mode. A negative value means there is no maximum.This field is present only if you enable enhanced test mode feature in the account., defaults to None
        :type test_mode_max_docs: int, optional
        :param worker_elastic_scaling_threshold: worker_elastic_scaling_threshold, defaults to None
        :type worker_elastic_scaling_threshold: int, optional
        :param worker_max_execution_time: Maximum run time in milliseconds for Execution worker processes. For example, an accepted value is 30000. After this amount of time passes, a 522 HTTP status code message is returned to the client saying that the process exceeds the time limit and then cancels the process run. After you change this property value, you must restart the Runtime cloud cluster or Runtime., defaults to None
        :type worker_max_execution_time: int, optional
        :param worker_max_general_execution_time: Maximum run time in milliseconds for Execution worker processes that use Low\_Latency\_Debug mode. For example, an accepted value is 60000. After you change this property value, you must restart the Runtime cloud cluster or Runtime., defaults to None
        :type worker_max_general_execution_time: int, optional
        :param worker_max_queued_executions: Maximum number of extra processes that can queue in a single Execution worker, when the maximum number of processes is running. If specified, this value must be a positive number. After you change this property value you must restart the Runtime cloud cluster or Runtime., defaults to None
        :type worker_max_queued_executions: int, optional
        :param worker_max_running_executions: The maximum number of simultaneous processes that can run in a single Execution worker.After you change this property value, you must restart the Runtime cloud cluster or Runtime., defaults to None
        :type worker_max_running_executions: int, optional
        :param worker_queued_execution_timeout: Maximum time that a job in the queue can wait to run. After this amount of time, the job fails with a time-out exception.For example, an accepted value is 0:10:00. After you change this property value you must restart the Runtime cloud cluster or Runtime., defaults to None
        :type worker_queued_execution_timeout: int, optional
        """
        if account_disk_usage is not SENTINEL:
            self.account_disk_usage = account_disk_usage
        if as2_workload is not SENTINEL:
            self.as2_workload = self._enum_matching(
                as2_workload, As2Workload.list(), "as2_workload"
            )
        if atom_input_size is not SENTINEL:
            self.atom_input_size = atom_input_size
        if atom_output_overflow_size is not SENTINEL:
            self.atom_output_overflow_size = atom_output_overflow_size
        if atom_working_overflow_size is not SENTINEL:
            self.atom_working_overflow_size = atom_working_overflow_size
        if cloud_account_execution_limit is not SENTINEL:
            self.cloud_account_execution_limit = cloud_account_execution_limit
        if cloud_account_execution_warning_offset is not SENTINEL:
            self.cloud_account_execution_warning_offset = (
                cloud_account_execution_warning_offset
            )
        if container_id is not SENTINEL:
            self.container_id = container_id
        if download_runnerlogs is not SENTINEL:
            self.download_runnerlogs = download_runnerlogs
        if enable_account_data_archiving is not SENTINEL:
            self.enable_account_data_archiving = enable_account_data_archiving
        if enable_atom_worker_warmup is not SENTINEL:
            self.enable_atom_worker_warmup = enable_atom_worker_warmup
        if flow_control_parallel_process_type_override is not SENTINEL:
            self.flow_control_parallel_process_type_override = self._enum_matching(
                flow_control_parallel_process_type_override,
                FlowControlParallelProcessTypeOverride.list(),
                "flow_control_parallel_process_type_override",
            )
        if http_request_rate is not SENTINEL:
            self.http_request_rate = http_request_rate
        if http_workload is not SENTINEL:
            self.http_workload = self._enum_matching(
                http_workload, HttpWorkload.list(), "http_workload"
            )
        if listener_max_concurrent_executions is not SENTINEL:
            self.listener_max_concurrent_executions = listener_max_concurrent_executions
        if max_connector_track_docs is not SENTINEL:
            self.max_connector_track_docs = max_connector_track_docs
        if min_numberof_atom_workers is not SENTINEL:
            self.min_numberof_atom_workers = min_numberof_atom_workers
        if numberof_atom_workers is not SENTINEL:
            self.numberof_atom_workers = numberof_atom_workers
        if queue_incoming_message_rate_limit is not SENTINEL:
            self.queue_incoming_message_rate_limit = queue_incoming_message_rate_limit
        if session_id is not SENTINEL:
            self.session_id = session_id
        if status_code is not SENTINEL:
            self.status_code = status_code
        if test_mode_max_doc_bytes is not SENTINEL:
            self.test_mode_max_doc_bytes = test_mode_max_doc_bytes
        if test_mode_max_docs is not SENTINEL:
            self.test_mode_max_docs = test_mode_max_docs
        if worker_elastic_scaling_threshold is not SENTINEL:
            self.worker_elastic_scaling_threshold = worker_elastic_scaling_threshold
        if worker_max_execution_time is not SENTINEL:
            self.worker_max_execution_time = worker_max_execution_time
        if worker_max_general_execution_time is not SENTINEL:
            self.worker_max_general_execution_time = worker_max_general_execution_time
        if worker_max_queued_executions is not SENTINEL:
            self.worker_max_queued_executions = worker_max_queued_executions
        if worker_max_running_executions is not SENTINEL:
            self.worker_max_running_executions = worker_max_running_executions
        if worker_queued_execution_timeout is not SENTINEL:
            self.worker_queued_execution_timeout = worker_queued_execution_timeout
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .oftp_partner_group_type import OftpPartnerGroupType
from .oftp_send_options_info import OftpSendOptionsInfo


@JsonMap(
    {
        "oftp_partner_group": "OFTPPartnerGroup",
        "oftp_send_options": "OFTPSendOptions",
        "partner_group_id": "partnerGroupId",
    }
)
class OftpSendOptions(BaseModel):
    """OftpSendOptions

    :param oftp_partner_group: oftp_partner_group
    :type oftp_partner_group: OftpPartnerGroupType
    :param oftp_send_options: oftp_send_options
    :type oftp_send_options: OftpSendOptionsInfo
    :param partner_group_id: partner_group_id, defaults to None
    :type partner_group_id: str, optional
    """

    def __init__(
        self,
        oftp_partner_group: OftpPartnerGroupType,
        oftp_send_options: OftpSendOptionsInfo,
        partner_group_id: str = SENTINEL,
        **kwargs,
    ):
        """OftpSendOptions

        :param oftp_partner_group: oftp_partner_group
        :type oftp_partner_group: OftpPartnerGroupType
        :param oftp_send_options: oftp_send_options
        :type oftp_send_options: OftpSendOptionsInfo
        :param partner_group_id: partner_group_id, defaults to None
        :type partner_group_id: str, optional
        """
        self.oftp_partner_group = self._define_object(
            oftp_partner_group, OftpPartnerGroupType
        )
        self.oftp_send_options = self._define_object(
            oftp_send_options, OftpSendOptionsInfo
        )
        if partner_group_id is not SENTINEL:
            self.partner_group_id = partner_group_id
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment_role import EnvironmentRoleService
from ...models import (
    EnvironmentRole,
    EnvironmentRoleBulkResponse,
    EnvironmentRoleBulkRequest,
    EnvironmentRoleQueryResponse,
    EnvironmentRoleQueryConfig,
)


class EnvironmentRoleServiceAsync(EnvironmentRoleService):
    """
    Async Wrapper for EnvironmentRoleServiceAsync
    """

    def create_environment_role(
        self, request_body: EnvironmentRole = None
    ) -> Awaitable[Union[EnvironmentRole, str]]:
        return to_async(super().create_environment_role)(request_body)

    def get_environment_role(self, id_: str) -> Awaitable[Union[EnvironmentRole, str]]:
        return to_async(super().get_environment_role)(id_)

    def delete_environment_role(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_environment_role)(id_)

    def bulk_environment_role(
        self, request_body: EnvironmentRoleBulkRequest = None
    ) -> Awaitable[Union[EnvironmentRoleBulkResponse, str]]:
        return to_async(super().bulk_environment_role)(request_body)

    def query_environment_role(
        self, request_body: EnvironmentRoleQueryConfig = None
    ) -> Awaitable[Union[EnvironmentRoleQueryResponse, str]]:
        return to_async(super().query_environment_role)(request_body)

    def query_more_environment_role(
        self, request_body: str
    ) -> Awaitable[Union[EnvironmentRoleQueryResponse, str]]:
        return to_async(super().query_more_environment_role)(request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .comp_diff_attribute import CompDiffAttribute


@JsonMap({"comp_diff_attribute": "CompDiffAttribute"})
class CompDiffElement(BaseModel):
    """CompDiffElement

    :param comp_diff_attribute: comp_diff_attribute, defaults to None
    :type comp_diff_attribute: List[CompDiffAttribute], optional
    :param ignored: ignored, defaults to None
    :type ignored: bool, optional
    :param name: The tag name of the element that you want to target for the comparative diff results., defaults to None
    :type name: str, optional
    :param ordered: ordered, defaults to None
    :type ordered: bool, optional
    :param parent: The parent element for the element that you want to configure for the comparative diff results., defaults to None
    :type parent: str, optional
    """

    def __init__(
        self,
        comp_diff_attribute: List[CompDiffAttribute] = SENTINEL,
        ignored: bool = SENTINEL,
        name: str = SENTINEL,
        ordered: bool = SENTINEL,
        parent: str = SENTINEL,
        **kwargs,
    ):
        """CompDiffElement

        :param comp_diff_attribute: comp_diff_attribute, defaults to None
        :type comp_diff_attribute: List[CompDiffAttribute], optional
        :param ignored: ignored, defaults to None
        :type ignored: bool, optional
        :param name: The tag name of the element that you want to target for the comparative diff results., defaults to None
        :type name: str, optional
        :param ordered: ordered, defaults to None
        :type ordered: bool, optional
        :param parent: The parent element for the element that you want to configure for the comparative diff results., defaults to None
        :type parent: str, optional
        """
        if comp_diff_attribute is not SENTINEL:
            self.comp_diff_attribute = self._define_list(
                comp_diff_attribute, CompDiffAttribute
            )
        if ignored is not SENTINEL:
            self.ignored = ignored
        if name is not SENTINEL:
            self.name = name
        if ordered is not SENTINEL:
            self.ordered = ordered
        if parent is not SENTINEL:
            self.parent = parent
        self._kwargs = kwargs
